/**
 * ================================
 * Forms BI Bean Graph Integration
 * ================================
 * 
 * @author   Frank Nimphius
 * @version  1.1
 * 
 * history of changes 
 * ================================
 * 1.  November 2002 initial creation
 * 1.  December 2002 added getProperty() method for property GET_DELIMITER to obtain the delimiter 
 *     string from a get_custom_property() call
 * 18. December , bug fixes 2708349,2708328
 *    
 *    added getProperty() method for property COLUMNCOUNT to obtain the number of 
 *    columns from a get_custom_property() call
 *    
 *    
*/ 

package oracle.forms.demos.bigraph;

// Java classes
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.lang.NumberFormatException;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JTextPane;
import javax.swing.SwingConstants;

import oracle.dss.graph.BaseGraphComponent;
import oracle.dss.graph.Graph;
import oracle.dss.graph.GraphBackground;
import oracle.dss.graph.LegendArea;
import oracle.dss.graph.Series;
import oracle.dss.util.DefaultErrorHandler;

import oracle.forms.handler.IHandler;
import oracle.forms.properties.ID;
import oracle.forms.ui.CustomEvent;
import oracle.forms.ui.VBean;


/**
 * <p><b>FormsGraph</b> is a charting client for Oracle Forms, including the BI Bean Graph 
 * component to Forms. This sample does not expose all of the functionality provided by the BI Graph Bean, 
 * but as much as was possible with <I>Oracle Graphics</I>, the previous charting solution used within 
 * Forms.</p>
 * <p><b>FormsGraph</b> can be used in multiple instances, each running in a Bean Container managed
 * by Forms. Mouse Click events are passed back to Forms firing the <I>when-custom-item-event</I> trigger
 * so that interactive charts as well as drill down and master/detail relationships are within the
 * control of the Forms application developer.</p>
 * 
 * <p><b>version 1.0 Review</b></p>
 * 
*/
public class FormsGraph extends VBean

{
  private String sDelimiter = ",";                      // default delimiter for the data passed to the graph
  private int iSelectCol     = 0;                       // defines the Bar or pie that is selected when first
  private int iSelectRow     = 0;                       // rendering the chart. Can be overwritten
  private int mChart_height  = 400;                     // default value for chart height. 
  private int mChart_width   = 400;                     // default value for chart width. 
  private int mGraphType = Graph.BAR_VERT_CLUST;        // default graph type is a simple vertical bar graph
  private int mViewListenerCount = 0;                   // only one view listener shall get implemented
  private int mSeparateFrameXPos=10;                    // xPos of the separate Graph frame
  private int mSeparateFrameYPos=10;                    // yPos of the separate Graph frame   
  private boolean recoverGraphToVBean=false;            // this flag is set when the Graph component
                                                        // in the Forms Bean component is replaced by a panel
                                                        // due to no data available in the graph
  private int AlignRight =  SwingConstants.RIGHT; 
  private int AlignLeft =   SwingConstants.LEFT;
  private int AlignCenter=  SwingConstants.CENTER;
  protected boolean showGraphAsSeries = false;
  private boolean dataSuccess = false;                  // flag indicating if data row operation was successful
  private boolean bDebugMode = false;                   // debug true/false
  private String mWindowTitle = "Forms - BI Graph";     // title of separate frame
  private String noDataFoundTxt = "Graph Contains No Data!"; // message shown when graph does not contain data
  private String debugPrefix ="";
  private IHandler mHandler;
  private Component mBeanBox;
  private Vector vGraphData = null;                     // vector storing the data rows passed from Forms
  private Graph m_graph = null;
  private Series graphSeries = null;                    // handler to the graph series
  private mViewMouseListener instanceVMListener=null;   // reacts to mouse events performed on a graph
  private localRelationalData lrd=null;
  private JTextPane NoDataFoundPanel = new JTextPane();
  private ImageIcon emptyGraphImage=null;

  // identifier used to define the data returned by a mouseclick 
  // on the Graph
  public final int allData =     0;
  public final int dataLabel =   1;
  public final int dataColumn =  2;
  public final int dataValue =   3;
  public final int dataPrimKey=  4;
  public final int noData     =  5;

  // by default returns null
  protected int returnValueSelection = noData;
  
   /**
   * Forms property used to to add a data row in the Graph. 
   * If Graph Data exists, then the new row is appended. A row contains columns data which are separated 
   * by a defined delimiter. The default delimiter is ",", custom delimiters can be set using the Forms 
   * property <code>SET_DELIMITER </code> in a call to SET_CUSTOM_PROPERTY(). Used with SET_CUSTOM_PROPERTY() 
   * from the Form.
   * 
   * Syntax: <column_label><delimiter><row_label><delimiter><value><delimiter><primaryKey> e.g
   *        'SAL,SMITH,1200,7750'
   *      
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'ADD_ROWDATA','SAL,SMTITH,1200);</code>  
   * <p>or, if a primarey key needs to be passed</p>
   * <code>set_custom_property('BeanArea',1,'ADD_ROWDATA','SAL,SMTITH,1200,7765);</code>          
   */
  protected static final ID pAddRowData               = ID.registerProperty("ADD_ROWDATA");

/**
 *  Show the data in the Graph clearing out existing data
 *  <p><b>Forms Example:</b></p>
 *  <code>set_custom_property('BeanArea',1,'ADD_DATA_TO_GRAPH','');</code>
 */
 protected static final ID pAddDataToGraph            = ID.registerProperty("ADD_DATA_TO_GRAPH");

/**
 * Property registered to determine the alignment for the title, subtitle and footnote text. The allowed values are "LEFT",
 * "RIGHT" and "CENTER". One property call can be used to set all title alignments. The following attribute syntax sets the title first, then 
 * the subtitle and at lst the footnote 'title=CENTER,subtitle=LEFT,footnote=CENTER'. The delimiter by default it a comma (',') if not set other using
 * the SET_DELIMITER property. Values can be omitted. For example. If the alignment should be 
 * set for the title and subtitle only, then the following syntax is valid
 * 'title=CENTER,subtitle=LEFT'
 * 
 * <p><b>Forms Example:</b></p>
 * <code>set_custom_property('BeanArea',1,'ALIGN_TITLE_TEXT','title=CENTER,subtitle=LEFT');</code>
 */
  protected static final ID pAlignTitleText           = ID.registerProperty("ALIGN_TITLE_TEXT");

  /**
   * Forms property registered to reset the Forms Graph. Clearing the Graph empties the 
   * data cache of the graph. All data will be removed from the internal Graph store
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'CLEAR_GRAPH','');</code>
   * 
   */
  protected static final ID pClearGraph               = ID.registerProperty("CLEAR_GRAPH");

 /**
  * Forms registered property to enable Debug message (TRUE/FALSE). Arguments passed are either 
  * "TRUE" or "FALSE" to switch on/off verbose debug messages to system out.
  * <p><b>Forms Example:</b></p>
  * <code>set_custom_property('BeanArea',1,'DEBUG','TRUE');</code>
  */
  protected static final ID pDebug                    = ID.registerProperty("DEBUG");
   
  /**
   * Property registered to disable/enable tooltips. By default no tooltips are shown, the following 
   * Attributes are supported if provided with the set_custom_property() call in Forms
   * "ALL"    - show detail information (default)
   * "LABELS" - labels only
   * "VALUES" - show data values only
   * "NONE"   - show no tooltips
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'ENABLE_TOOLTIPS','ALL');</code>
   */
  protected static final ID pEnableTooltips            = ID.registerProperty("ENABLE_TOOLTIPS");

  /**
  * Forms registered property to explode a specific slice, for example '3,30' to explode the fourth
  * slice by 30 points.
  * 
  * Specifies how far the slices for the series are exploded. Exploded slices are pulled apart from 
  * the rest of the pie that they belong to. The values range from 0 to 100.
  * 
  *  <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'EXPLODE_PIESLICE','2,60');</code>
  */
  protected static final ID pExplodePieSLice            = ID.registerProperty("EXPLODE_PIESLICE");

 
   /**
   * Forms property registered to define the xy-position of the separate frame. The value passed is 
   * defined in Pixel and must be passed as integer. The attribute value is of kind '<x-val><delimiter>
   * <y-val>, where the delimiter by default is a comma
   * Used with SET_CUSTOM_PROPERTY() from the Form
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'FRAME_POS','100,200');</code>
   */
  protected static final ID pFramePos            = ID.registerProperty("FRAME_POS");

   /**
   * Forms property registered to define the type of the Graph displayed. 
   * 
   * HORIZONTAL_BAR,HORIZONTAL_BAR_2Y,VERTICAL_BAR,VERTICAL_BAR_2Y, VERTICAL_STACKED_BAR,HORIZONTAL_STACKED_BAR,
   * VERTICAL_PERCENT_BAR, HORIZONTAL_PERCENT_BAR,VERTICAL_LINE_GRAPH, HORIZONTAL_LINE_GRAPH, RING_BAR
   * VERTICAL_STACKED_LINE_GRAPH, HORIZONTAL_STACKED_LINE_GRAPH,VERTICAL_LINE_SPLIT_GRAPH,VERTICAL_AREA_GRAPH, VERTICAL_PERCENT_AREA_GRAPH,
   * VERTICAL_STACKED_AREA_GRAPH,PIE_GRAPH,PIE_BAR_GRAPH,MULTI_PIE_GRAPH,STOCK_HIGHLOW_GRAPH,MULTI_PIE_PROPORTIONAL_GRAPH,MULTI_RING_PROPORTIONAL_GRAPH,
   * MULTI_RING_GRAPH,3D_BAR_GRAPH, 3D_AREA_GRAPH, RADAR 
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'GRAPHTYPE','PIE_BAR_GRAPH');</code>
   */
  protected static final ID pGraphType            = ID.registerProperty("GRAPHTYPE");

   /**
   * Forms property registered to make the Graph Display show in a separate frame. This is to be used
   * with Forms Bean Containers of a <b>minimum</b> size of 1x1 Pixel. If the Bean Container is chosen 
   * bigger, then the container will be empty if running the Graph in an external frame.
   * Used with SET_CUSTOM_PROPERTY() from the Form. The Frame title can be passed as an argument with the 
   * call
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SHOW_GRAPH_IN_FRAME','');</code>
   */  
  protected static final ID pGraphInFrame        = ID.registerProperty("SHOW_GRAPH_IN_FRAME"); // displays Graph in separate frame

   /**
   * Forms property registered to make the Graph display inside of the Forms Bean Container. The 
   * Graph resizes itself to fit into the size defined by the Bean Container. The window title can be
   * set by the attribute passed with the Forms call to SET_CUSTOM_PROPERTY()
   * Used with SET_CUSTOM_PROPERTY() from the Form
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SHOW_GRAPH','');</code>
   */
  protected static final ID pGraph                = ID.registerProperty("SHOW_GRAPH");          // displays Graph in the Forms Bean Container

   /**
   * Forms property registered to hide the external frame 
   * Used with SET_CUSTOM_PROPERTY() from the Form
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'HIDE_FRAME','');</code>
   */
  protected static final ID pHideFrame            = ID.registerProperty("HIDE_FRAME");  

  /**
   * Property registered to hide the graph. Useful if no data is displayed in the graph. The 
   * BI Graph Bean uses internal test data if no data is set by the relational API. In a master
   * detail situation this is the case if no detail data is selected. To avoid confusion the 
   * Graph can be hidden by this command. The Graphs automatically displays when data is provided
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'HIDE_GRAPH','');</code>
   */
  protected static final ID pHideGraph            = ID.registerProperty("HIDE_GRAPH");

  /**
   * property registered to enable/disable mouse action [true/false]. Setting mouse action to false
   * produces less network traffic because the Forms client does not check with the server for a response
   * to a mouse event (click)
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'MOUSEACTION','false');</code>
   */
  protected static final ID pMouseAction        = ID.registerProperty("MOUSEACTION");

  /**
   * property registered to update a column or slice in a graph. This allows a partial graph 
   * update without refreshing all data in the graph
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'MODIFY_ROW_DATA','ColumnName,RowName,OldValue,NewValue');</code> 
   */
  protected static final ID pModifyData         = ID.registerProperty("MODIFY_ROW_DATA");
   
  /**
   * Property registered to position the legend area in a screen. Use TOP, LEFT, RIGHT, BOTTOM or
   * AUTO as an argument when calling set_custom_property
   * 
   * The default behavior of the Graph Bean though is to auto layout the legend according to the 
   * space left
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'POSITION_LEGEND','LEFT');</code>
   */
  protected static final ID pPositionLegendArea   = ID.registerProperty("POSITION_LEGEND");
  
  /**
   * Forms property registered to set the graph background color. The color must be passed as sRGB color 
   * in a comma separated string, with the specified red, green, and blue values in the range 
   * (0 - 255), e.g. '255,255,255' for  white. Basic colors: black, blue, cyan, darkGray, gray,
   * green, lightGray, magenta, orange, pink, red, white can be passed in clear text. 
   * Used with SET_CUSTOM_PROPERTY() from the Form
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SET_BACKGROUND','120,234,123');</code>
   */
  protected static final ID pSetBackgroundColor   = ID.registerProperty("SET_BACKGROUND");
   
   /**
   * Forms property registered to set the delimiting character for data passed as string.&nbsp;The default delimiter is a comma
   * but there are situations where it must be changed, like when setting the title, which includes the color
   * Used with SET_CUSTOM_PROPERTY() from the Form
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SET_DELIMITER','#');</code>
   */
  protected static final ID pSetDelimiter         = ID.registerProperty("SET_DELIMITER");

  /**
   * property registered to show hide value markers in Line graphs.&nbsp;Use 'true' to switch this feature on
   * and 'false' to switch it off. 
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SET_LINEGRAPH_MARKER','true');</code>
   * 
   */
  protected static final ID pSetLineGraphMarkers       = ID.registerProperty("SET_LINEGRAPH_MARKERS");
  
  /**
   * property registered to logarithmic scale the Y and X Axis.&nbsp;Pass the axis identifier, true or false to symbolize 
   * whether logarithmic scale is enabled and the base. The delimiter must match the currently defined delimiter,
   * which is by default ','.
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SET_SCALED_LOGARITHMIC','Y,true,2');</code>
   * <p> or</p>
   * <code>set_custom_property('BeanArea',1,'SET_SCALED_LOGARITHMIC','X,true');</code>
   * 
   * <p> In above example the Y axis is defined as scaled logarithmic with a base of 2. The 
   * base is provided as a double value. Note that there is always only one axis that shows the 
   * value of the graph. The ability to set the X-Axis to a logarithmic scale is for all horizontal
   * graphs, while setting logarithmic scale to the Y axis is for all vertical graphs
   * </p>
   */
  protected static final ID pSetScaledLogarithmic      = ID.registerProperty("SET_SCALED_LOGARITHMIC");

  /**
  * Property registered to set the graph's baseline to a value other than 0.&nbsp;Use this property if you want 
  * to show the variance a graph column has from a specific reference point. The new baseline value is passed as 
  * a double value from Forms
  * <p><b>Forms Example:</b></p>
  * <code>set_custom_property('BeanArea',1,'SET_GRAPH_BASELINE','2000');</code>
  * <p> to reset </p>
  * <code>set_custom_property('BeanArea',1,'SET_GRAPH_BASELINE','0');</code>
  */
  protected static final ID pSetBaseline               = ID.registerProperty("SET_GRAPH_BASELINE");
  
  /**
  * <b>DEPRECATED for use in set_custom_property() in version 1.1</b> use GET_DELIMITER in a get_custom_property() 
  * call instead! - property registered to retrieve the current delimiter String.&nbsp;The response is a Forms custom 
  * event firing, passing the new delimiter string as an argument
  * 
  * <p><b>Forms Example:</b></p>
  * <code>set_custom_property('BeanArea',1,'GET_DELIMITER','');</code>
  */
  protected static final ID pGetDelimiter         = ID.registerProperty("GET_DELIMITER");

  /**
  *  Property registered to show/hide the grid in a graph.&nbsp;The default value is set to true, use false to hide the graph from showing
  * <p><b>Forms Example:</b></p>
  * <code>set_custom_property('BeanArea',1,'SHOW_GRID','false');</code>  
  */
  protected static final ID pShowGrid             = ID.registerProperty("SHOW_GRID");

  /**
  * property registered to remove a data row currently displayed in the graph.&nbsp;If the graph contains
  * a primary key then this also gets deleted from the data store. The data row to delete is identified 
  * by the match of columns name, row name and value
  * <p><b>Forms Example:</b></p>
  * <code>set_custom_property('BeanArea',1,'REMOVE_DATA','ColumnName,RowName,value');</code>  
  */
  protected static final ID pRemoveData       = ID.registerProperty("REMOVE_DATA");
  /**
   * properties registered to hide the graph title if set
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'HIDE_TITLE','');</code>  
   */
  protected static final ID pRemoveTitle          = ID.registerProperty("HIDE_TITLE");

  /**
   * properties registered to hide the sub_titles if set
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'HIDE_SUBTITLE','');</code>  
   */
  protected static final ID pRemoveSubTitle       = ID.registerProperty("HIDE_SUBTITLE");

  /**
   * properties registered to hide the footer if set
   <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'HIDE_FOOTER','');</code>  
   */
  protected static final ID pRemoveFooter         = ID.registerProperty("HIDE_FOOTER");

  /**
   * properties registered to hide the x axis label if set
   * 
   *<p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'HIDE_X_TITLE','');</code>  
   */
  protected static final ID pRemoveX_Title        = ID.registerProperty("HIDE_X_TITLE");  

  /**
   * properties registered to hide the y axis label title if set
   *<p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'HIDE_Y_TITLE','');</code>     
   */
  protected static final ID pRemoveY_Title        = ID.registerProperty("HIDE_Y_TITLE");  
  
  /**
  * property registered to define return values of a mouseclick on a graph.&nbsp;Allowed arguments are
  * "ALL", "ROWLABEL", "COLUMNLABEL","VALUE","PRIMARY_KEY". The primary is returned only if it was passed
  * using the ADD_ROW_DATA property
  * <p><b>Forms Example:</b></p>
  * <p>--Forms example for a master/detail Graph. Code added to the WHEN-CUSTOM-EVENT Trigger</p>
  * <p>-- on the PJC.MASTERGRAPH </p>
  * <code>set_custom_property('BeanArea',1,'RETURN_VALUES_ON_CLICK','ALL');</code>  
  * <p><b> to retrieve the result in Forms:</b></p>
  * <p><code>
  * DECLARE
	*
	* <p>&nbsp;&nbsp;eventName varchar2(30) := :system.custom_item_event;</p>
	* <p>&nbsp;&nbsp;tempString varchar2(100);</p>
	* <p>&nbsp;&nbsp;eventValues ParamList;</p>
	* <p>&nbsp;&nbsp;eventValueType number;</p>
	* <p></p>	
  * <p>BEGIN</p>
	* <p></p>
	* <p>IF (eventName='GRAPH_ACTION') THEN</p>
	* <p>&nbsp;&nbsp;		eventValues := get_parameter_list(:system.custom_item_event_parameters);</p>
	* <p>&nbsp;&nbsp;		get_parameter_attr(eventValues,'GRAPH_INFO',eventValueType, tempString);</p>
	* <p>&nbsp;&nbsp;		FORMSGRAPHSAMPLE.clearData('PJC.DETAILGRAPH');</p>
	* <p>&nbsp;&nbsp;		FORMSGRAPHSAMPLE.populateDetailGraphData('PJC.DETAILGRAPH',tempString,',');</p>
	* <p>ELSE</p>
	* <p>&nbsp;&nbsp;		 	null;</p>
	* <p>END IF;</p>
  * <p>END;</p>
  * </code></p>
  */
  protected static final ID pReturnValues         = ID.registerProperty("RETURN_VALUES_ON_CLICK");

  /**
   * Property registered to display a scrollbar on the Graph if needed. Set value to true for
   * enabling the scrollbar (default) or false to disable the scrollbar. The default is false as
   * this feature is only required when too many columns are shown that cannot be displayed in 
   * the current graph
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SCROLLBAR','true');</code> 
   */
  protected static final ID pScrollBar        = ID.registerProperty("SCROLLBAR");
  
/**
 * property registered to define the depth of a graph and the orientation of this depth. 
 * Specify both values in one call, where the fist value determines the depth and the second the radius. 
 * The delimiter is the actual delimiter defined or ',' if choosing the default. To se a depth of 20 pixel and 
 * a radius of 10 degrees, set the attribute value of this Forms property to '20,10' (if the default
 * delimiter id used). Allowable rage of degrees are from 0 - 180
 * <p><b>Forms Example:</b></p>
 * <code>set_custom_property('BeanArea',1,'SET_DEPTH','30,75');</code> 
 */
  protected static final ID pSetDepth         = ID.registerProperty("SET_DEPTH");

  /**
   * property registered to set a prefix string to the debug message produced by the bean.&nbsp;This
   * is useful if running multiple instances of this bean in one Forms application.&nbsp;In this case 
   * it is useful to know which bean wrotes the output when looking in the Java console
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SET_DEBUG_PREFIX','Bean1');</code> 
   */
  protected static final ID pSetDebugPrefix  = ID.registerProperty("SET_DEBUG_PREFIX");
  /**
   * property registered to set the width and hight for the external Graph frame. The size is passed
   * in as a string representing two integers delimited by the delimiter used (default ',');
   * e.g. 200,300 where the first value always represents the height.
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SET_FRAME_HEIGHT_WIDTH','200,300');</code> 
   */
  protected static final ID pSetFrameWidthAndHeight = ID.registerProperty("SET_FRAME_HEIGHT_WIDTH");
  
  /**
   *  property registered to set the graph title, font, font style, font size and color,  
  *   &nbsp;expecting the following string format <I>title string,color,fontstyle,font size,fontname</I>. 
  *   For example to set a Subtitle to "Hello World", the Font to TimesNewRoman, the font 
  *   style to bold [b] and italic [i] and the color to red the string shows:
  *   
  *   'Hello World,red,bi,10,TimesNewRoman'
  *   
  *   The color can be specified as color name or as a sRGB value in a range from (0..255). Values
  *   can be omitted from right to left. 
  *   
  * <p><b>Forms Example:</b></p>
  * <p><code>
  * <p>BEGIN</p>
  * <p>&nbsp;&nbsp;-- the color is defined with comma separated rgb values. Thus the default delimiter</p>
  * <p>&nbsp;&nbsp;-- must be changed temporarily. Alos set the font type to bold and italic (bi)</p>
  * <p>&nbsp;&nbsp;SET_CUSTOM_PROPERTY('PJC.SIMPLEGRAPH',1,'SET_DELIMITER','#');</p>
  * <p>&nbsp;&nbsp;SET_CUSTOM_PROPERTY('PJC.SIMPLEGRAPH',1,'SET_TITLE','Hello World#120,234,120#bi#12');</p>
  * <p>&nbsp;&nbsp;SET_CUSTOM_PROPERTY('PJC.SIMPLEGRAPH',1,'SET_DELIMITER',',');</p>
  * <p>END;</p> </code></p>
  */
  protected static final ID pSetTitle         = ID.registerProperty("SET_TITLE");

 /**
  *  Property registered to set the message shown when no data is provided for the graph to render. This is the case 
  *  whenever a master-detail relationship does not have detail data If the message text is about to contain commas 
  *  then define a new delimiter before and reset it after.
  * <p><b>Forms Example:</b></p>
  * <code>set_custom_property('BeanArea',1,'SET_NO_DATA_FOUND','Sorry - we are out of data today');</code> 
  */
  protected static final ID pSetNoDataFoundMessage = ID.registerProperty("SET_NO_DATA_FOUND");
  
  /**
   * property registered to set the border and background color of the legend shown in a graph.
   * The color can be provided as an rgb value e.g. 120,234,22 or as a name e.g. red, orange, green
   * The first value passed is for the border, the second for the background. 
   * '120,200,222|223,223,222' If one or both setting should be transparent, then pass
   * 'TRANSPARENT' as a color name. E.g. 'Transparent|123,334,22'. Note that in this case the default
   * delimiter must be changed since the color coding requires ',' as a delimiter
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'SET_LEGEND_BORDER','blue,yellow');</code> 
  */
  protected static final ID pSetLegendBorder  = ID.registerProperty("SET_LEGEND_BORDER");
  
 /**
   * Used with SET_CUSTOM_PROPERTY() from the Form, retrieves the number of columns in a graph 
   * Column means the group of data displayed: Columns e.g. are SAL, COMM, REVENUE
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'COLUMNCOUNT','');</code> 
   * <p><b>In a when-custom-item-event trigger</b></p>
   * <p><code>
   * DECLARE
   * &nbsp;&nbsp;	eventName varchar2(30) := :system.custom_item_event;
   * &nbsp;&nbsp;	tempString varchar2(100);
   * <p></p>	
   * <p>BEGIN</p>
   * <p>&nbsp;&nbsp; IF (eventName='RETURNED_COLUMN_NUMBER') THEN </p>
   * <p>&nbsp;&nbsp;eventValues := get_parameter_list(:system.custom_item_event_parameters);</p>
   * <p>&nbsp;&nbsp;get_parameter_attr(eventValues,'GRAPH_INFO',eventValueType, tempString);</p>
   * <p>&nbsp;&nbsp; ... </p>
   * <p>END;</p>
   * </code></p>
  */
  protected static final ID pSeriesCount            = ID.registerProperty("COLUMNCOUNT");

  /**
  *   property registered to set the graph Sub title, font, font style, font size and color. 
  *   The following string format is expected
  *   <p>"title string,color,font style,font size,font"</p>
  *   <p>
  *   For example to set a Subtitle to "Hello World", the Font to TimesNewRoman, the font 
  *   style to bold [b] and italic [i] and the color to red the string shows:</p>
  *   
  *   'Hello World,red,bi,10,TimesNewRoman'
  *   
  *   <p>The color can be specified as color name or as a sRGB value in a range from (0..255). Values
  *   can be omitted from right to left. Passing color values as RGB values may require switching the
  *   default delimiter as the comma must be used to set RGB values</p>
  *   
  * <p><b>Forms Example:</b></p>
  * <code>set_custom_property('BeanArea',1,'SET_SUBTITLE','Hello World,red,bi,10,TimesNewRoman');</code> 
  * <p><b> or </b></p>
  * <p><code>set_custom_property('BeanArea',1,'SET_DELIMITER','#');</code></p>
  * <p><code>set_custom_property('BeanArea',1,'SET_SUBTITLE','Hello&nbsp;World#120,234,120#bi#10#TimesNewRoman');</code></p> 
  * <p><code>set_custom_property('BeanArea',1,'SET_DELIMITER',',');</code></p> 
  */
  protected static final ID pSetSubTitle         = ID.registerProperty("SET_SUBTITLE");

    /**
  *   property registered to set the graph Footer title, font, font style, font size and color. 
  *   The following string format is expected<p>
  *  "title string,color,font style,font size,font"</p>
 *   <p>For example to set a Subtitle to "Hello World", the Font to TimesNewRoman, the font 
  *  style to bold [b] and italic [i] and the color to red the string shows:</p>
  *   
  *   'Hello World,red,bi,10,TimesNewRoman'
  *   
  *   <p>The color can be specified as color name or as a sRGB value in a range from (0..255). Values
  *   can be omitted from right to left. Passing color values as RGB values may require switching the
  *   default delimiter as the comma must be used to set RGB values</p>
  *   
  *   <p><b>Forms Example:</b></p>
  *   <code>set_custom_property('BeanArea',1,'SET_FOOTER','Hello World,red,bi,10,TimesNewRoman');</code> 
  *   <p><b> or </b></p>
  *   <p><code>set_custom_property('BeanArea',1,'SET_DELIMITER','#');</code></p>
  *   <p><code>set_custom_property('BeanArea',1,'SET_FOOTER','Hello&nbsp;World#120,234,120#bi#10#TimesNewRoman');</code></p> 
  *   <p><code>set_custom_property('BeanArea',1,'SET_DELIMITER',',');</code></p> 
  */
    protected static final ID pSetFootnote       = ID.registerProperty("SET_FOOTER");

  /**
   * Property registered to set the color of the Plot Area. If not set then the area is chosen to 
   * be white. The color must be passed as RGB color in a comma separated string, with the specified 
   * red, green, and blue values in the range (0 - 255), e.g. '255,255,255' for  white. Basic colors: 
   * black, blue, cyan, darkGray, gray,green, lightGray, magenta, orange, pink, red, white can be passed 
   * in clear text. 
   * <p><b>Forms Example:</b></p>
  * <p><code>set_custom_property('BeanArea',1,'SET_PLOT_AREA_COLOR','255,255,255');</code></p>
  *
  */
  protected static final ID pSetPlotAreaColor            = ID.registerProperty("SET_PLOT_AREA_COLOR");
  /**
   * property registered to set the background color of the title text field. Normally you want that color 
   * be set to the same value then the background, but sometimes having this appear in a different color may
   * make sense.
   * The color must be passed as RGB color in a comma separated string, with the specified red, green, and blue 
   * values in the range (0 - 255), e.g. '255,255,255' for  white. Basic colors: black, blue, cyan, darkGray, gray,
   * green, lightGray, magenta, orange, pink, red, white can be passed in clear text. 
   * <p><b>Forms Example:</b></p>
  * <p><code>set_custom_property('BeanArea',1,'SET_TITLE_BACKGROUND','255,255,255');</code></p>
   */
  protected static final ID pSetTitleBackgroundColor     = ID.registerProperty("SET_TITLE_BACKGROUND");

  /**
   * property registered to set the background color of the subtitle title text field. Normally you want that color 
   * be set to the same value then the background, but sometimes having this appear in a different color may
   * make sense.
   * The color must be passed as RGB color in a comma separated string, with the specified red, green, and blue 
   * values in the range (0 - 255), e.g. '255,255,255' for  white. Basic colors: black, blue, cyan, darkGray, gray,
   * green, lightGray, magenta, orange, pink, red, white can be passed in clear text. 
   * <p><b>Forms Example:</b></p>
  * <p><code>set_custom_property('BeanArea',1,'SET_SUBTITLE_BACKGROUND','255,255,255');</code></p>
   */
  protected static final ID pSetSubTitleBackgroundColor  = ID.registerProperty("SET_SUBTITLE_BACKGROUND");

  /**
   * property registered to set the background color of the subtitle footer text field. Normally you want that color 
   * be set to the same value then the background, but sometimes having this appear in a different color may
   * make sense.
   * The color must be passed as RGB color in a comma separated string, with the specified red, green, and blue 
   * values in the range (0 - 255), e.g. '255,255,255' for  white. Basic colors: black, blue, cyan, darkGray, gray,
   * green, lightGray, magenta, orange, pink, red, white can be passed in clear text. 
   * <p><b>Forms Example:</b></p>
  * <p><code>set_custom_property('BeanArea',1,'SET_FOOTER_BACKGROUND','255,255,255');</code></p>
   */
  protected static final ID pSetFooterBackgroundColor    = ID.registerProperty("SET_FOOTER_BACKGROUND");

  /**
  *  property registered to set the x-Axis title
   * <p><b>Forms Example:</b></p>
  *  <p><code>set_custom_property('BeanArea',1,'SET_X_LABEL','Hello&nbsp;World&nbsp;X');</code></p>  
  */ 
    protected static final ID pSetXLabel         = ID.registerProperty("SET_X_LABEL");
  /**
  *  property registered to set the y-Axis title
  * <p><b>Forms Example:</b></p>
  *  <p><code>set_custom_property('BeanArea',1,'SET_Y_LABEL','Hello&nbsp;World&nbsp;Y');</code></p>  
  */ 
    protected static final ID pSetYLabel        = ID.registerProperty("SET_Y_LABEL");

   /**
   * Forms property registered to show columns as rows and rows a columns in the Graph. If specifying the 
   * argument passed as "true" then the columns will show as series data and the rows as group data. To switch
   * this presentation back to normal mode, call the same property again and pass "false" as an argument.
   * 
   * <p><b>Forms Example:</b></p>
   * <p><code>set_custom_property('BeanArea',1,'SHOW_COLUMNS_AS_ROWS','true');</code></p>
   */
   protected static final ID pShowColumnsAsRows    = ID.registerProperty("SHOW_COLUMNS_AS_ROWS"); 

   /**
   * Forms property registered to show the external frame Graph. This feature is used when running the Graph
   * in an external frame to the Forms window. It is to unhide a frame that previously was hidden.
   * 
   * <p><b>Forms Example:</b></p>
   * <p><code>set_custom_property('BeanArea',1,'SHOW_FRAME','');</code></p>
   */ 
  protected static final ID pShowFrame            = ID.registerProperty("SHOW_FRAME");   

   /**
   * Forms property registered to show/hide the labels shown on the x and y axis. Labels
   * are shown by default, to hide labels pass a value of false with the argument of the
   * Forms property. Use the following syntax: 'x=false,y1=true' to show labels for the
   * y1 axis but to hide labels for the x axis. You can omit the outer right value if
   * no change is needed e.g. 'x=true'. The delimiter by default is a ',' if
   * not overwritten using the SET_DELIMITER property
   * 
   * <p><b>Forms Example:</b></p>
   * <p><code>set_custom_property('BeanArea',1,'SHOW_LABELS','x=false,y=true');</code></p>
   */
  protected static final ID pShowLabels            = ID.registerProperty("SHOW_LABELS"); 

  /**
   * Forms property registered to show/hide the graph legend. Attribute value to be passed
   * 'true' to show the legend (default) 'false' to hide
   * 
   * <p><b>Forms Example:</b></p>
   * <p><code>set_custom_property('BeanArea',1,'SHOW_LEGEND','true');</code></p>
   */
  protected static final ID pShowLegend            = ID.registerProperty ("SHOW_LEGEND");


  /*
    CUSTOM ITEM EVENTS
    ==================
  */
  
  /** 
   * Event registered to inform about the number of series (columns) shown in a graph. The event is caught in a 
   * when-custom-item event trigger on the Bean Container
   * <p><b>Forms Example:</b></p>
   * <p><code>
   * ...
   * <p>IF (eventName='RETURNED_COLUMN_NUMBER') THEN</p>
	 * <p>&nbsp;&nbsp;eventValues := get_parameter_list(:system.custom_item_event_parameters);</p>
	 * <p>&nbsp;&nbsp;get_parameter_attr(eventValues,'GRAPH_INFO',eventValueType, tempString);</p>
   * <p>&nbsp;&nbsp;...</p>
   * <p></code></p>
  */
  public static final ID eGetSeriesCount         = ID.registerProperty("RETURNED_COLUMN_NUMBER");

  /**
   * Custom Item Event raised by mouse click on the graph. Note that this event wont work if 
   * pMouseAction is set to false. The event is caught in a when-custom-item event trigger on 
   * the Bean Container
   * 
   * <p><b>Forms Example:</b></p>
   * <p><code></p>
   * <p>...</p>
   * <p>IF (eventName='GRAPH_ACTION') THEN</p>
	 * <p>&nbsp;&nbsp;eventValues := get_parameter_list(:system.custom_item_event_parameters);</p>
	 * <p>&nbsp;&nbsp;get_parameter_attr(eventValues,'GRAPH_INFO',eventValueType, tempString);</p>
   * <p>&nbsp;&nbsp;...</p>
   * <p></code></p>
   */
  public static final ID eGraphAction           = ID.registerProperty("GRAPH_ACTION");

  /**
   * Property registered to obtain information about graph component that the user clicked on 
   */
  public static final ID pGraphInfo             = ID.registerProperty("GRAPH_INFO");

  /**
   * Property registered to get information about the delimiter
   */
  protected static final ID eGetDelimiter         = ID.registerProperty("CURRENT_DELIMITER");
  /**
   * property registered to obtain the delimiter sign
   */
  protected static final ID pDelimiterInfo        = ID.registerProperty("DELIMITER_INFO");

  //=========================
  // END OF FORMS PROPERTIES
  //=========================
  private JFrame jf = null;
  
  public FormsGraph()
  {

    super();
    // instantiate the BI Bean Graph

    m_graph = new Graph();
    
    /*====================
     * SET GRAPH DEFAULT 
     * ==================*/
     
    // show rows as in the relational world
    m_graph.setDataRowShownAsASeries(false);
    // have the Nean managing the layout of the Graph
    m_graph.setAutoLayout(Graph.AL_ALWAYS);
    // The default type is set to bar chart
    m_graph.setGraphType(mGraphType);
    // the graph is first shown when data is provided
    m_graph.setVisible(false);
    // hide tooltips
    m_graph.setMarkerTooltipType(Graph.TLT_NONE);
    // disable debug information sent from the graph
    DefaultErrorHandler deh = new DefaultErrorHandler();
    deh.setDebugMode(DefaultErrorHandler.SHOW_NONE);
    m_graph.addErrorHandler(deh);


    // instantiate the data store for this graph
    lrd = new localRelationalData(this);

    /* Test Data allowing you to run this Java Bean in Forms without providing 
     * Data Forms. use this data for developing new features and functionality
     * or for tracing problems*/
     
    /*========================================================================== 
    dataSuccess=lrd.addRelationalDataRow("Europe,Sales,2000,1",sDelimiter);
    dataSuccess=lrd.addRelationalDataRow("Europe,Provisions,2000,2",sDelimiter);
    dataSuccess=lrd.addRelationalDataRow("USA,Sales,4000,3",sDelimiter);
    dataSuccess=lrd.addRelationalDataRow("USA,Provisions,5000,4",sDelimiter);
    dataSuccess=lrd.addRelationalDataRow("Europe,Netto,2200.12,5",sDelimiter);
    dataSuccess=lrd.addRelationalDataRow("USA,Netto,30000,6",sDelimiter);
    dataSuccess=lrd.addRelationalDataRow("USA,Brutto,7000,7",sDelimiter);
    dataSuccess=lrd.addRelationalDataRow("Europe,Brutto,5000,8",sDelimiter);

    m_graph.setLocalRelationalData(lrd.getRelationalData());    
    ============================================================================*/
    /* Test Data End */

    
    //************ set mouse interaction to on **********
    instanceVMListener = new mViewMouseListener(m_graph, this,lrd);
    mViewListenerCount=1;
    //***************************************************

    m_graph.addViewMouseListener(instanceVMListener);
  }

  /**
   * Implementation of IView interface. Init is called when the bean is first instanciated
   * 
   * @see oracle.forms.ui.IView
   */
  public void init(IHandler handler)
  {
    mHandler = handler;
    super.init(handler);
    super.add(m_graph);     
  }

   /**
   * Implementation of IView interface which sets a requested property to a given value
   * 
   * @param _ID property to be set.
   * @param _object value of the property id.
   * @return      true if the property could be set, false otherwise.
   * @see oracle.forms.ui.IView
   */
  public boolean setProperty(ID _ID, Object _object)
  {
  try 
  {  
          
      /*=====================
       * ADD ROW DATA
       ======================*/
      if (_ID == pAddRowData)
      {
        String sGraphDataRow = null;
        
        // get String from Forms Object
        if (_object!=null&&(String)_object!="") 
        {
          sGraphDataRow = (String) _object;

          // check if delimiter is used correctly
          if (sGraphDataRow.indexOf(sDelimiter) > 0)
          {
            /* Pass string to localRelationalData class. in here the string
             * is parsed and ignored if it doesn't contain 4 parameters and
             * if a number exception occurs. The data parameters are of tyle 
             * label, label, double and finally integer
             */
              if (!lrd.addRelationalDataRow(sGraphDataRow,sDelimiter))
              {
                DebugMessage("ADD_ROW_DATA: "+sGraphDataRow+" not accepted");
              }
            return true;  
          }
          else
          {
            DebugMessage("ADD_ROWDATA: No valid string delimiter found. Expected \""+sDelimiter+"\"");
            return true;
          }
        }
        else 
        {
          DebugMessage("ADD_ROWDATA: No data passed");
          return true;
        }
        
      }
      
      /*=====================
      * ADD_DATA_TO_GRAPH
       ======================*/
      else if (_ID == pAddDataToGraph)
      {
        DebugMessage("ADD_DATA_TO_GRAPH - setting data for display");
        
        ArrayList al = lrd.getRelationalData();
        // handle no data passed
        if (al.size()==0) 
        {
            // decide on the response in the UI
            noGraphDataFoundHandler();   
        } else{
          // if the graph component previously have been removed from the 
          // Graph then it needs to be added back there
          if(recoverGraphToVBean)
          {
                // remove component that was added insetad of the graph
                super.remove(0);
                // add graph for display
                super.add(m_graph);
                // reset flag
                recoverGraphToVBean=false;         
          }
          m_graph.setVisible(true);       
          m_graph.setLocalRelationalData(al);
        }
        
        return true;
      }
      
      /*=====================
       * ALIGN TITLE TEXT
       ======================*/
      else if (_ID == pAlignTitleText)
      {
          String _sObject = (String) _object;
          _sObject = handleTokenNullvaluesInStartAndEnd(_sObject);
          
          StringTokenizer st = new StringTokenizer(_sObject,sDelimiter);
          int tokenLength = st.countTokens();
          String firstToken= "",secondToken = "", thirdToken="";
        
          DebugMessage("Property ALIGN_TITLE_TEXT has received "+tokenLength+" tokens and a value of "+(String)_object);

        if(tokenLength>0){
          firstToken=(String)st.nextElement();
          String tValue = firstToken.substring(firstToken.toLowerCase().indexOf("title=")+6);
          DebugMessage ("ALIGN_TITLE_TEXT: FIRST TOKEN: "+tValue);
            if ("right".equalsIgnoreCase(tValue)) 
              {
                m_graph.getDataviewTitle().setHorizontalAlignment(AlignRight);
              } 
            else if ("left".equalsIgnoreCase(tValue)) 
              {
                m_graph.getDataviewTitle().setHorizontalAlignment(AlignLeft);
              }
            else // center
            {
                m_graph.getDataviewTitle().setHorizontalAlignment(AlignCenter);
            }
          }

          if (tokenLength >1){
            secondToken = (String) st.nextElement();
            String tValue = secondToken.substring(secondToken.toLowerCase().indexOf("subtitle=")+9);
            DebugMessage ("ALIGN_TITLE_TEXT: SECOND TOKEN: "+tValue);
            
              if ("right".equalsIgnoreCase(tValue)) 
              {
                m_graph.getDataviewSubtitle().setHorizontalAlignment(AlignRight);
              } 
            else if ("left".equalsIgnoreCase(tValue)) 
              {
                m_graph.getDataviewSubtitle().setHorizontalAlignment(AlignLeft);
              }
            else // center
            {
                m_graph.getDataviewSubtitle().setHorizontalAlignment(AlignCenter);
            }
          }

          if (tokenLength >2){
            thirdToken = (String) st.nextElement();
            String tValue = thirdToken.substring(thirdToken.toLowerCase().indexOf("footnote=")+9);
            DebugMessage ("ALIGN_TITLE_TEXT: THIRD TOKEN: "+tValue);
            
              if ("right".equalsIgnoreCase(tValue)) 
              {
                m_graph.getDataviewFootnote().setHorizontalAlignment(AlignRight);
              } 
            else if ("left".equalsIgnoreCase(tValue)) 
              {
                m_graph.getDataviewFootnote().setHorizontalAlignment(AlignLeft);
              }
            else // center
            {
                m_graph.getDataviewFootnote().setHorizontalAlignment(AlignCenter);
            }
          }
        return true;
      }
      
      /*=====================
       * CLEAR GRAPH
      ======================*/
    else if (_ID==pClearGraph)
      {
        lrd.clearGraphData();           
        return true;
      }
  
      /*=====================
       * DEBUG
       ======================*/
      else if (_ID==pDebug)
      {
        if ("TRUE".equalsIgnoreCase((String)_object))
        {
          bDebugMode = true;
        }
        else if ("FALSE".equalsIgnoreCase((String)_object))
        {
          bDebugMode = false;
        }
        else 
        {
          // ignore
        }
        
        return true;
      }
    
      /*=====================
       * ENABLE_TOOLTIPS
       =====================*/ 
       else if (_ID == pEnableTooltips){
        if(_object!=null&&(String)_object!="") 
        {
          if ("ALL".equalsIgnoreCase((String) _object))
          {
            // this is the default setting and you don't have to do
            // anything for this
            
            DebugMessage("ENABLE_TOOLTIPS: setting all tooltips");
            m_graph.setMarkerTooltipType(Graph.MTT_VALUES_TEXT );
          }
           if ("LABELS".equalsIgnoreCase((String) _object))
          {
            DebugMessage("ENABLE_TOOLTIPS: setting label tooltips");
            m_graph.setMarkerTooltipType(Graph.MTT_TEXT );
            
          }
          else if ("VALUES".equalsIgnoreCase((String) _object))
          { 
            DebugMessage("ENABLE_TOOLTIPS: setting value tooltips");
            m_graph.setMarkerTooltipType(Graph.TLT_MEMBER);
          }
          else if ("NONE".equalsIgnoreCase((String) _object))
          { 
            DebugMessage("ENABLE_TOOLTIPS: disabling tooltips");
            m_graph.setMarkerTooltipType(Graph.TLT_NONE);
          }
          else
          {
            DebugMessage("ENABLE_TOOLTIPS: No valid attribute passed. Use ALL, LABELS,VALUES or NONE");
          }
          
        }
        else
        {
          DebugMessage("ENABLE_TOOLTIPS: No attribute value passed, thus ignoring");
        }
        return true;
       }
      /*=====================
       * EXPLODE SLICE
      ======================*/
      // moves the selected slice number away from the rest
      else if (_ID == pExplodePieSLice)
      {
      //_object contains a delimited String containing values, where 
      // the second value must be between 0 and 100. The first should not exceed
      // the number of graph columns available or be negative

      if (_object==null&&(String)_object=="") 
      {
        // go read the manual !
        DebugMessage("EXPLODE_PIESLICE requires attribute passed. please refer to the"+
                          " documentation on how to use this property.");
      }
      else
      {
        // some exceptions may be raised on the way, so we prep for it
        try 
        {

          StringTokenizer st = new StringTokenizer(((String)_object),sDelimiter);
          String[] val = new String[2];
          int tokenLength = st.countTokens();
          if (tokenLength != 2)
          {
            // go read the manual !
            DebugMessage("EXPLODE_PIESLICE requires attribute passed. please refer to the"+
                              " documentation on how to use this property.");
          }
          else
          {
            val[0] = (String) st.nextElement();
            val[1] = (String) st.nextElement();

            DebugMessage("Property EXPLODE SLICE: Attributes passed: "+ (String)_object);
            DebugMessage("After untokenizing String: series="+val[0]+" and explode rate="+val[1]);

            int Series=0;
            int ExpRate=0;

            Series =  (new Double(val[0])).intValue();
            ExpRate = (new Double(val[1])).intValue();

            // if you get here the there wasn't any numeric exceptions
            // check if series is below or over Graph range and adjust if necessary
            Series = (Series < 0? 0:Series > (m_graph.getSeriesObjectCount()-1)?(m_graph.getSeriesObjectCount()-1):Series);
            // check explode rate between 0 and 100, if not set to edge values
            ExpRate= (ExpRate<0?0:(ExpRate >100?100:ExpRate));       

            graphSeries = new Series(m_graph);
            graphSeries.setPieSliceExplode(ExpRate,Series);
          }              
        } 
         catch (NumberFormatException nfe)
          {
            DebugMessage("Property  EXPLODE_PIESLICE: Not a valid integer format passed");
            DebugMessage(nfe.getMessage());
          }

        catch (Exception ex) 
        {
          ex.printStackTrace();
        }
                
      }
      return true;
    }

      /*=====================
       * FRAME_POS
       ======================*/
      //Defines the x and y position of the sperarate graph frame
      
      else if (_ID == pFramePos)
        {
          
          if (_object!=null&&(String)_object!="") 
          {
            DebugMessage("FRAME_POS: trying to set position to "+(String) _object);
            StringTokenizer st = new StringTokenizer((String) _object,sDelimiter);
            int tokenLength = st.countTokens();
            String firstToken= "",secondToken = "";
            int xPos= 0, yPos=0;
            try{
              if (tokenLength > 0)
              {
                firstToken= (String)st.nextElement();
                xPos = new Double(firstToken).intValue();
                mSeparateFrameXPos = xPos;
              }
            if (tokenLength > 1)
            {
               secondToken= (String)st.nextElement();
               yPos = new Double(secondToken).intValue();
               mSeparateFrameYPos = yPos;
            }

            }
            catch (NumberFormatException nfe)
            {
             DebugMessage("FRAME_POS Exception: Data passed for x and y postion could not be converted to int");
             }
            catch (Exception e)
             {
              DebugMessage("FRAME_POS: Exception"+e.getMessage());
             }
           }
          else
          {
            DebugMessage("FRAME_POS: No positional attributes passed - Ignoring command");
          }
          return true;
        }

      /*=====================
       * GRAPH
       ======================*/
       // Adds the Graph component to the Bean Container
      else if (_ID==pGraph)
        {
          DebugMessage("GRAPH: adding to container");
          super.add(m_graph);
          
          // This is the relational look, where rows appear on teh x-axis while
          // columns show as groups in teh legend. This seems to be different in 
          // the BI world, where groups are shown on te x-axis by default.
          m_graph.setDataRowShownAsASeries(false);
          m_graph.getVolumeMarker().selectObject(0,0);
          return true;
        }

      /*=====================
       * GRAPH IN FRAME
       ======================*/
       // Creates a separate frame for the external Graph window
      else if (_ID==pGraphInFrame)
        {

             DebugMessage("GRAPH_IN_FRAME: adding to frame");
            mWindowTitle=( _object == null ? mWindowTitle : (String) _object);
            jf = new JFrame(mWindowTitle);
            jf.getContentPane().add(m_graph);
            jf.setLocation(mSeparateFrameXPos,mSeparateFrameYPos);
            jf.setSize(mChart_width,mChart_height);
            jf.addWindowListener(new WindowAdapter(){
                public void windowClosing(WindowEvent e)
                {
                        jf.hide();
                }
            });
            jf.hide();
            return true;
        }

      /*
       *=====================
       * GRAPH TYPE
       ======================*/
      else if (_ID == pGraphType)
        {
            DebugMessage("GRAPH_TYPE: trying to set graph type");
            mGraphType = (_object==null?mGraphType:getInternalGraphType(_object));
            m_graph.setGraphType(mGraphType);
            return true;
        }
      /*=====================
       * HIDE GRAPH
       * ===================*/
      else if(_ID == pHideGraph)
      {
        DebugMessage("HIDE_GRAPH");
        m_graph.setVisible(false);
        // in case that the graph was replaced by the No data found panel,
        // indicating that no data was provided for the graph
        NoDataFoundPanel.setVisible(false);
        
        return true;
      }
      /*=====================
       * HIDE FRAME
       ======================*/
      else if (_ID == pHideFrame)
        {
            DebugMessage("HIDE_FRAME: hiding");
            jf.hide();
            return true;
        }
    /*=====================
       * HIDE_TITLE
       * ===================*/
       else if (_ID == pRemoveTitle)
       {
        DebugMessage("HIDE_TITLE: hiding");
         m_graph.getDataviewTitle().setVisible(false);
         return true;
       }
      /*=====================
       * HIDE_SUBTITLE
       * ===================*/
       else if (_ID == pRemoveSubTitle)
       {
         DebugMessage("HIDE_SUBTITLE: hiding");
         m_graph.getDataviewSubtitle().setVisible(false);
         return true;
       }
      /*=====================
       * HIDE_FOOTER
       * ===================*/
       else if (_ID == pRemoveFooter)
       {
         DebugMessage("HIDE_FOOTER: hiding");
         m_graph.getDataviewFootnote().setVisible(false);
         return true;
       }

      /*=========================
       * HIDE X TITLE
       * ======================*/
       else if (_ID == pRemoveX_Title)
       {
         m_graph.getO1Title().setVisible(false);
         DebugMessage("HIDE_X_TITLE: set visible to false");
         return true;
       }

      /*=========================
       * HIDE Y TITLE
       * ======================*/
       else if (_ID == pRemoveY_Title)
       {
         m_graph.getY1Title().setVisible(false);
         DebugMessage("HIDE_Y_TITLE: set visible to false");
         return true;
       }

      /*=====================
       * MOUSE ACTION
       ======================*/
      else if (_ID==pMouseAction)
      { 
        if ("TRUE".equalsIgnoreCase((String)_object)){
        /*
         * only one instance of the mouse view listsner is allowed for this object. If there exists an
         * instance of this listener type, then all other registration attempts are ignored
        */
        mViewMouseListener[] existingListeners = (mViewMouseListener[]) m_graph.getListeners(mViewMouseListener.class);
     
         if (mViewListenerCount==0){        
              instanceVMListener = new mViewMouseListener(m_graph,this,lrd);
              DebugMessage("MOUSE_ACTION: Adding mouse listener");
              m_graph.addViewMouseListener(instanceVMListener);
              ++mViewListenerCount;
          }
        }

          else if ("FALSE".equalsIgnoreCase((String)_object)){
     
            // remove mouse listener when exists
            mViewMouseListener[] existingListeners = (mViewMouseListener[]) m_graph.getListeners(mViewMouseListener.class);    
            if (mViewListenerCount!=0){        
            DebugMessage("MOUSE_ACTION: Removing mouse listener");
            m_graph.removeViewMouseListener(instanceVMListener);
            mViewListenerCount=0;
            }
          }
          else 
          {
            // ignore
            DebugMessage("Property MOUSE_ACTION: "+(String) _object + " passed but TRUE or FALSE required. Ignoring command!");
          }
          return true;
      }     
      /*=======================
       * MODIFY_DATA
       * =====================*/
      else if (_ID == pModifyData)
      {
        if (_object!=null&&(String)_object!="") 
        {
          DebugMessage("MODIFY_DATA: Trying to update row in graph");
          if (lrd.ModifyData((String)_object,sDelimiter)){
            //refresh graph data
            m_graph.setLocalRelationalData(lrd.getRelationalData());
          }
          else
          {
            DebugMessage("MODIFY_DATA: No matching column/row found in graph. Statement ignored");
          }
        } else 
        {
          DebugMessage("MODIFY_DATA: No values passed with request. Statement ignored");
        }
        
      }
      /*========================
       * POSITION_LEGEND
       * =======================*/
       else if (_ID == pPositionLegendArea)
       {
         if (_object!=null&&(String)_object!="") 
         {
            DebugMessage("POSITION_LEGEN: position legend to "+(String)_object);
            if ("AUTO".equalsIgnoreCase((String) _object)) 
            {
              LegendArea legend = m_graph.getLegendArea();
              legend.setAutomaticPlacement(BaseGraphComponent.AP_ALWAYS);
            } else 
            {
              if ("LEFT".equalsIgnoreCase((String)_object)) 
              {
                // first, turn off automatic placement
                  LegendArea legend = m_graph.getLegendArea();
                  legend.setAutomaticPlacement(BaseGraphComponent.AP_NEVER);
                  // set the position to the left edge 
                  legend.setPosition(BaseGraphComponent.LAP_LEFT); 
                  
              } else if ("RIGHT".equalsIgnoreCase((String)_object)) 
              {
                // first, turn off automatic placement
                  LegendArea legend = m_graph.getLegendArea();
                  legend.setAutomaticPlacement(BaseGraphComponent.AP_NEVER);
                  // set the position to the right edge 
                  legend.setPosition(BaseGraphComponent.LAP_RIGHT); 
              }
              else if ("TOP".equalsIgnoreCase((String)_object)) 
              {
                // first, turn off automatic placement
                  LegendArea legend = m_graph.getLegendArea();
                  legend.setAutomaticPlacement(BaseGraphComponent.AP_NEVER);
                  // set the position to the top edge 
                  legend.setPosition(BaseGraphComponent.LAP_TOP); 
              }
               else if ("BOTTOM".equalsIgnoreCase((String)_object)) 
               {
                  // first, turn off automatic placement
                  LegendArea legend = m_graph.getLegendArea();
                  legend.setAutomaticPlacement(BaseGraphComponent.AP_NEVER);
                  // set the position to the bottom edge 
                  legend.setPosition(BaseGraphComponent.LAP_BOTTOM); 

               }
               else
               {
                 DebugMessage("POSITION_LEGEND:value "+(String) _object+" passed does not represent a valid argument");
               }
            }
            
          
         } else 
         {
           DebugMessage("POSITION_LEGEND: No value passed. Pass LEFT,TOP,BOTTOM,RIGHT,AUTO as an argument when calling set_custom_property()");
         }
         
       }
      /*=====================
       * RETURN VALUES
       ======================*/
      // define the type of data returned when clicking on a graph
      // allowed values are "ALL", "ROWLABEL", "COLUMNLABEL","CELLVALUE","PRIMARY_KEY"
      else if (_ID==pReturnValues)
      {
        if(_object!=null&&(String)_object!="") 
        {
          DebugMessage("RETURN VALUES: trying to set return value ....");
          if ("ALL".equalsIgnoreCase((String)_object))
          {
            returnValueSelection=allData;
            DebugMessage("RETURN VALUES: ... to ALL");
          }
          else if ("ROWLABEL".equalsIgnoreCase((String)_object))
          {
            returnValueSelection=dataLabel;
            DebugMessage("RETURN VALUES: ... to ROWLABEL");
          }
          else if ("COLUMNLABEL".equalsIgnoreCase((String)_object))
          {
            returnValueSelection=dataColumn;
            DebugMessage("RETURN VALUES: ... to COLUMNLABEL");
          }
          else if ("VALUE".equalsIgnoreCase((String)_object))
          {
            returnValueSelection=dataValue;
            DebugMessage("RETURN VALUES: ... to VALUE");
          }
          else if ("PRIMARY_KEY".equalsIgnoreCase((String)_object))
          {
            returnValueSelection=dataPrimKey;
            DebugMessage("RETURN VALUES: ... to PRIMARY_KEY");
          }
          else if ("NONE".equalsIgnoreCase((String)_object))
          {
            returnValueSelection=noData;
            DebugMessage("RETURN VALUES: ... to NONE");
          }
          else 
          {
            DebugMessage("Property RETURN_VALUES: wrong attribute passed. Statement ignored");
            DebugMessage("allowed: \"ALL\",\"ROWLABEL\",\"COLUMNLABEL\",\"CELLVALUE\"or \"PRIMARY_KEY\"");
          }
          
        }
        else 
        {
           DebugMessage("Property RETURN_VALUES: no attribute passed. Statement ignored");
        }
        return true;
      }

      /*====================
       * SCROLLBAR
       * ==================*/

       else if (_ID == pScrollBar)
      {
         if (_object!=null&&(String)_object!="") 
         {
            DebugMessage("SCROLLBAR: setting value to "+(String) _object);
            if ("TRUE".equalsIgnoreCase((String) _object)) 
            {
               m_graph.setScrollbarPresenceGroups(Graph.SP_AS_NEEDED);
            } else if ("FALSE".equalsIgnoreCase((String) _object))
            {
              // disabling scrollbars
              m_graph.setScrollbarPresenceGroups(Graph.SP_NEVER);
            }
            else
            {
              DebugMessage("SCROLLBAR: value "+(String)_object+" does not contain TRUE or FALSE");
            }
            
         } else 
         {
          DebugMessage("SCROLLBAR: no value associated with command");
         }
         return true;
       }

       /*=====================
        * SET_LEGEND_BORDER
        * ====================*/
        else if (_ID==pSetLegendBorder)
        {
          String bgColor="";
          String brdColor = "";
          Color bgC = null;
          Color brdC = null;
          
          if (_object!=null&&(String)_object!="") 
          {
            String _sObject = (String)_object;
            _sObject = handleTokenNullvaluesInStartAndEnd(_sObject);
            StringTokenizer st = new StringTokenizer(_sObject,sDelimiter);
            int ct = st.countTokens();
            DebugMessage("SET_LEGEND_BORDER: Token count = "+ct);
            for (int i=0; i<ct;i++)
            {
              // get/set colors for border and background
              switch(i)
              {
                case 0: brdColor = (String)st.nextElement();
                        if ("TRANSPARENT".equalsIgnoreCase(brdColor))
                        {
                          m_graph.getLegendArea().setBorderTransparent(true);
                        }
                        else{
                          DebugMessage("SET_LEGEND_BORDER: Border color value = "+brdColor);
                          brdC=colorCodeRegistry.getColorCode(brdColor);
                          m_graph.getLegendArea().setBorderColor(brdC);
                        }
                        break;
                case 1: bgColor = (String)st.nextElement();
                        if ("TRANSPARENT".equalsIgnoreCase(bgColor))
                        {
                           m_graph.getLegendArea().setFillTransparent(true);
                        }
                        else{
                          DebugMessage("SET_LEGEND_BORDER: Background color value = "+bgColor);
                          bgC=colorCodeRegistry.getColorCode(bgColor);
                          m_graph.getLegendArea().setFillColor(bgC);
                        }
                        break;
                default: 
              }
              
            }
            
          } else 
          {
            DebugMessage("SET_LEGEND_BORDER: No value passed for setting the border color");
          }
          
        }
      /*=====================
       * SERIES COUNT
       ======================*/
      else if (_ID==pSeriesCount)
      {
        // retrieve graph column count for further usage
        DebugMessage("SERIES COUNT requested");
        // if rows shown as series then row number = column number
        if (showGraphAsSeries){
          returnSeriesCount(m_graph.getRowCount());
        }
        else
        {
          returnSeriesCount(m_graph.getColumnCount());
        }
        return true;
      }
             
       /*=====================
       * SET BACKGROUND COLOR
       ======================*/
      // the background color is passed either in a comma delimited string of RBG vlues in 
      // a range of (o...255) , or as a color name, e.g. red
      else if (_ID == pSetBackgroundColor)
      {
            if (_object!=null&&(String)_object!="") {
              Color col = colorCodeRegistry.getColorCode((String)_object);
              if (col != null)
              {
                DebugMessage("Property SET_BACKGROUND_COLOR: setting " +(String)_object+ " as a new background color");               
                GraphBackground gb = new GraphBackground(m_graph);
                gb.setBackground(col);
              }
              else
              {
                DebugMessage("Property SET_BACKGROUND_COLOR: "+(String)_object+" passed is not a valid color name or RGB value");               
              }
              return true;
            }
            else
            {
              DebugMessage("SET_BACKGROUND: no color specified"); 
              return true;
            }
      }

      /*=====================
       * SET PLOT AREA COLOR
       ======================*/
      // the background color of the Plot areais passed either in a comma delimited string of RBG vlues in 
      // a range of (o...255) , or as a color name, e.g. red
      else if (_ID == pSetPlotAreaColor)
      {
            if (_object!=null&&(String)_object!="") {
              Color col = colorCodeRegistry.getColorCode((String)_object);
              if (col != null)
              {
                DebugMessage("Property SET_PLOT_AREA_COLOR: setting " +(String)_object+ " as a new plot area color");               
                m_graph.getPlotArea().setFillColor(col);
              }
              else
              {
                DebugMessage("Property SET_PLOT_AREA_COLOR: "+(String)_object+" passed is not a valid color name or RGB value");               
              }
              return true;
            }
            else
            {
              DebugMessage("SET_PLOT_AREA_COLOR: no color specified"); 
              return true;
            }
      }

      /*===========================
       * SET_TITLE_BACKGROUND_COLOR
       * =========================*/

       else if (_ID==pSetTitleBackgroundColor)
       {
         if (_object!=null&&(String)_object!="") {
              Color col = colorCodeRegistry.getColorCode((String)_object);
              if (col != null)
              {
                DebugMessage("Property SET_TITLE_BACKGROUND: setting " +(String)_object+ " as a new background color");               
                m_graph.getDataviewTitle().setBackground(col);
              }
              else
              {
                DebugMessage("Property SET_TITLE_BACKGROUND: "+(String)_object+" passed is not a valid color name or RGB value");               
              }
              return true;
            }
            else
            {
              DebugMessage("SET_TITLE_BACKGROUND: no color specified"); 
              return true;
            }
       }
      /*===========================
       * SET_SUBTITLE_BACKGROUND_COLOR
       * =========================*/

       else if (_ID==pSetSubTitleBackgroundColor)
       {
         if (_object!=null&&(String)_object!="") {
              Color col = colorCodeRegistry.getColorCode((String)_object);
              if (col != null)
              {
                DebugMessage("Property SET_SUBTITLE_BACKGROUND: setting " +(String)_object+ " as a new background color");               
                m_graph.getDataviewSubtitle().setBackground(col);
              }
              else
              {
                DebugMessage("Property SET_SUBTITLE_BACKGROUND: "+(String)_object+" passed is not a valid color name or RGB value");               
              }
              return true;
            }
            else
            {
              DebugMessage("SET_SUBTITLE_BACKGROUND: no color specified"); 
              return true;
            }
       }
      /*===========================
       * SET_FOOTER_BACKGROUND_COLOR
       * =========================*/

       else if (_ID==pSetFooterBackgroundColor)
       {
         if (_object != null){
              Color col = colorCodeRegistry.getColorCode((String)_object);
              if (col != null)
              {
                DebugMessage("Property SET_FOOTER_BACKGROUND: setting " +(String)_object+ " as a new background color");               
                m_graph.getDataviewFootnote().setBackground(col);
              }
              else
              {
                DebugMessage("Property SET_FOOTER_BACKGROUND: "+(String)_object+" passed is not a valid color name or RGB value");               
              }
              return true;
            }
            else
            {
              DebugMessage("SET_FOOTER_BACKGROUND: no color specified"); 
              return true;
            }
       }
      /*======================
       * SET_DEPTH 
       ======================*/
      else if (_ID==pSetDepth)
      {
        if (_object!=null&&(String)_object!="") 
        {
          String _sObject = (String) _object;
          _sObject = handleTokenNullvaluesInStartAndEnd(_sObject);
          StringTokenizer st = new StringTokenizer(_sObject,sDelimiter);
          int tokenLength = st.countTokens();
          String firstToken= "",secondToken = "";
          DebugMessage("SET_DEPTH: tokens = "+firstToken+" , "+secondToken);
          firstToken= (String) st.nextElement();
          if(tokenLength >1){
            secondToken=(String) st.nextElement();
          }

          try
          {
           // set first token as depth value and second token as radius
           if (m_graph.getGraphType() == Graph.PIE)
           {
             // in a pie there is a depth value and a rotation angle
             m_graph.setPieDepth(new Double(firstToken).intValue());
             m_graph.setPieRotation(new Double(secondToken).intValue());
           }
           else
           {
              // all other graphs have a depth and a depth angle
              m_graph.setDepthRadius(new Double(firstToken).intValue());
              m_graph.setDepthAngle(new Double(secondToken).intValue());
           }

          }
          catch (NumberFormatException nfe)
          {
            DebugMessage("SET_DEPTH: Could not turn String value into integer - "+firstToken+" , "+secondToken);
          }
          
        }
        else 
        {
          DebugMessage("SET_DEPTH: No values passed");
        }
        return true;
      }
      /*=====================
       * SET DELIMITER
       ======================*/
      else if (_ID == pSetDelimiter)
        {
            DebugMessage("Property SET_DELIMITER: Trying to set delimiter value to "+(String)_object); 
            setDelimiter((_object==null?sDelimiter:(String)_object));
            return true;
        }

      /*===================================
       * SET_DEBUG_PREFIX
       * =================================*/
       else if (_ID == pSetDebugPrefix)
       {
         debugPrefix = (String) _object+" ==>";
         return true;
       }
      /*====================================
       * SET EXTERNAL FRAME WIDTH AND HEIGHT
       * ===================================*/
        else if (_ID == pSetFrameWidthAndHeight)
        {
          int frWidth =0;
          int frHeight=0;
          
          if (_object!=null&&(String)_object!="") 
          {
            try{
            String _sObject = (String)_object;
            _sObject = handleTokenNullvaluesInStartAndEnd(_sObject);
            StringTokenizer st = new StringTokenizer(_sObject,sDelimiter);
            int tokenLength = st.countTokens();
            frHeight = new Integer((String)st.nextElement()).intValue();
            if (tokenLength >1)
            {
              frWidth = new Integer((String)st.nextElement()).intValue();
              mChart_width = frWidth;
            }

              mChart_height = frHeight;
            }
            catch (NumberFormatException nfe)
            {
              DebugMessage ("pSetFrameWidthAndHeight: "+(String)_object+" does not contain valid integer values");
            }
          } else 
          {
            DebugMessage("pSetFrameWidthAndHeight: No sizing value provided for external frame - ignoring statement");
          }
          
        }
      /*=====================
       * SET TITLE
       ======================*/
        else if (_ID == pSetTitle)
        {
          if (_object!=null&&(String)_object!="") 
          {
          DebugMessage("SET TITLE retrieved string "+(String) _object);
          Object[] mo = getTitleFromString((String)_object);
          if (mo.length >0){
              m_graph.getDataviewTitle().setVisible(true);
              DebugMessage("Set Title Text: "+(String) mo[0]);        
              m_graph.getDataviewTitle().setText((String) mo[0]);
          }
           if (mo.length >1){
            DebugMessage("Set Title Font: "+((Font) mo[1]).getFontName());
            DebugMessage("Set Title Font Style: "+((Font) mo[1]).getStyle());
            m_graph.getDataviewTitle().setFont((Font)   mo[1]);
            m_graph.repaint();
           }
          if (mo.length >2){
              m_graph.getDataviewTitle().setForeground((Color)mo[2]);
          }
          m_graph.getDataviewTitle().setHorizontalAlignment(AlignCenter);
          }
          else
          {
            DebugMessage("Property SET_TITLE: No attributes passed");
          }
         return true; 
        }

      /*=====================
       * SET NO DATA FOUND MESSAGE
       * ====================*/
        else if (_ID == pSetNoDataFoundMessage){
                noDataFoundTxt= (String) _object;
        }
        
      /*=====================
       * SET SUBTITLE
       ======================*/
        else if (_ID == pSetSubTitle)
        {
        if (_object!=null&&(String)_object!="") 
          {
          Object[] mo = getTitleFromString((String)_object);
          m_graph.getDataviewSubtitle().setVisible(true);
          m_graph.getDataviewSubtitle().setText((String) mo[0]);
          m_graph.getDataviewSubtitle().setFont((Font)   mo[1]);
          m_graph.getDataviewSubtitle().setForeground((Color)mo[2]);
          m_graph.getDataviewSubtitle().setVisible(true);
          m_graph.getDataviewSubtitle().setHorizontalAlignment(AlignCenter);
          }
          else
          {
            DebugMessage("Property SET_SUBTITLE: no attributes passed");
          }          
          return true;
        }

      /*=====================
       * SET FOOTER
       ======================*/
        else if (_ID == pSetFootnote)
        {
          if (_object!=null&&(String)_object!="") 
          {
            Object[] mo = getTitleFromString((String)_object);
            m_graph.getDataviewFootnote().setVisible(true);
            m_graph.getDataviewFootnote().setText((String) mo[0]);
            m_graph.getDataviewFootnote().setFont((Font)   mo[1]);
            m_graph.getDataviewFootnote().setForeground((Color)mo[2]);
            m_graph.getDataviewFootnote().setVisible(true);
            m_graph.getDataviewFootnote().setHorizontalAlignment(AlignCenter);
          }
          else
          {
            DebugMessage("Property SET_FOOTNOTE: no attributes passed");
          }
          return true;
        }

      /*=====================
       * SET_X_TITLE
       =====================*/ 
      else if (_ID==pSetXLabel)
      {
        DebugMessage("Setting X_TITLE to '"+(String) _object+"'");
        m_graph.getO1Title().setVisible(true);
        m_graph.getO1Title().setText((String)_object);
        return true;
      }
      /*=====================
       * SET_Y_TITLE
       =====================*/ 
      else if (_ID==pSetYLabel)
      {
        DebugMessage("Setting Y_TITLE to '"+(String) _object+"'");
        m_graph.getY1Title().setVisible(true);
        m_graph.getY1Title().setText((String)_object);
        return true;
      }
      
      /*=====================
       * SHOW FRAME
       ======================*/
      else if (_ID== pShowFrame)
        {
            jf.show();
            return true;
        } 

     
      /*=====================
       * SHOW COLUMNS AS ROWS
       ======================*/
       // shows the columns as rows and the rows as columns
       
      else if (_ID== pShowColumnsAsRows)
        {
            if (_object!=null&&(String)_object!="") 
            {
              if ("true".equalsIgnoreCase((String)_object))
              {
                m_graph.setDataRowShownAsASeries(true);
                showGraphAsSeries=true;
                m_graph.getDataFilter().refresh();
              }
              else if ("false".equalsIgnoreCase((String)_object))
              {
                m_graph.setDataRowShownAsASeries(false);
                showGraphAsSeries=false;
                m_graph.getDataFilter().refresh();
              }
              else
              {
                // ignore
                DebugMessage("Property SHOW_COLUMNS_AS_ROWS attribute is not true nor false: Ignored");
              }
            }
            
            return true;
        } 

      /*********************
       * SHOW/HIDE GRID
       * ===================*/
      else if(_ID == pShowGrid)
             {
            if (_object!=null&&(String)_object!="") 
            {
              if ("true".equalsIgnoreCase((String)_object))
              {
                  m_graph.getY1MajorTick().setVisible(true);
                  m_graph.getO1MajorTick().setVisible(true);
              }
              else if ("false".equalsIgnoreCase((String)_object))
              {
                  m_graph.getY1MajorTick().setVisible(false);
                  m_graph.getO1MajorTick().setVisible(false);
              }
              else
              {
                // ignore
                DebugMessage("Property SHOW_GRID attribute is not true nor false: Ignored");
              }
            }
            
            return true;
        } 

      /*=====================
       * REMOVE_DATA
       * ===================*/
       else if (_ID==pRemoveData)
       {
         if (_object!=null&&(String)_object!="") 
         {
           if(lrd.RemoveData((String) _object, sDelimiter))
           {
             // refresh data shown in the graph
             m_graph.setLocalRelationalData(lrd.getRelationalData());
           }
           else
           {
             DebugMessage("REMOVE_DATA: No row matches search criteria!");
           }
         } else 
         {
          DebugMessage("REMOVE_DATA: No data passed, command ignored!");
         }
         return true;
       }
       
      /*=====================
       * SHOW LABELS
       =====================*/
       else if (_ID==pShowLabels)
       {
         String _sObject = (String) _object;
         _sObject = handleTokenNullvaluesInStartAndEnd(_sObject);
         StringTokenizer st = new StringTokenizer(_sObject,sDelimiter);
          int tokenLength = st.countTokens();
          String firstToken= "",secondToken = "";
        
          DebugMessage("Property SHOW_LABELS has received "+tokenLength+" tokens and a value of "+(String)_object);

        if(tokenLength>0){
          firstToken=(String)st.nextElement();
          String txValue = firstToken.substring(firstToken.indexOf("x=")+2);
          DebugMessage ("SHOW_LABELS: FIRST TOKEN: "+txValue);
            if ("false".equalsIgnoreCase(txValue)) 
              {
                m_graph.getX1Axis().setVisible(false);
              } 
            else 
              {
                m_graph.getX1Axis().setVisible(true);
              }
          }
          if (tokenLength >1){
            secondToken = (String) st.nextElement();
            String ty1Value = secondToken.substring(secondToken.indexOf("y1=")+3);
            DebugMessage ("SHOW_LABELS: SECOND TOKEN: "+ty1Value);
            
            if ("false".equalsIgnoreCase(ty1Value)) 
              {
                m_graph.getY1Axis().setVisible(false);
              } 
            else 
              {
                m_graph.getY1Axis().setVisible(true);
              }
          }
         
        return true;
       }

      /*=====================
       * SHOW_LEGEND 
       ======================*/
      else if(_ID==pShowLegend)
      {
        DebugMessage("SHOW_LEGEND: "+(String) _object);
        if (_object!=null&&(String)_object!="") 
        {
          if ("true".equalsIgnoreCase((String)_object))
          {
            DebugMessage("SHOW_LEGEND: value of true received");
            m_graph.getLegendArea().setVisible(true);
          }
          else if("false".equalsIgnoreCase((String)_object))
          {
            DebugMessage("SHOW_LEGEND: value of false received");
            m_graph.getLegendArea().setVisible(false);
          }
          else
          {
            DebugMessage("SHOW_LEGEND: neither true or false received as a value");
          }
        }
        else
        {
          DebugMessage("SHOW_LEGEND: No value passed");
        }
        return true;
      }
      /*=======================
       * SET_LINEGRAPH_MARKER
       * ======================*/
      else if(_ID == pSetLineGraphMarkers)
      {
        if (_object!=null && (String)_object!="") 
        {
          DebugMessage("SET_LINEGRAPH_MARKER: received "+(String) _object+" as an argument");
          if ("TRUE".equalsIgnoreCase((String)_object)) 
          {
            m_graph.setMarkerDisplayed(true);
          } else if ("FALSE".equalsIgnoreCase((String)_object)) 
          {
            m_graph.setMarkerDisplayed(false);
          }
          else
          {
           DebugMessage("SET_LINEGRAPH_MARKER: Use 'true' or 'false' as an argument with this call"); 
          }
          
          
        } else 
        {
          DebugMessage("SET_LINEGRAPH_MARKER: No data is passed with call to property. Allowed values are 'true' or 'false'");
        }
        
        return true;
      }
      
      /*========================
       * SET_SCALED_LOGARITHMIC
       * =======================*/
       else if (_ID == pSetScaledLogarithmic)
       {      
         if (_object != null&&(String)_object!="") 
         {
           String _sObject = (String) _object;
          _sObject = handleTokenNullvaluesInStartAndEnd(_sObject);
           StringTokenizer st = new StringTokenizer(_sObject,sDelimiter);
           int tokenLength = st.countTokens();
           String axis="";
           String enable="false";
           // unrealistic initial value
           double base=0;

           // parse string of arguments
           for (int i = 0; i < tokenLength && i<3; i++) 
           {
              switch(i)
              {
              // axis is Y or X
              case 0: axis = (String)   st.nextElement();
                      break;
              // enable id true or false
              case 1: enable = (String) st.nextElement();
                      break;
              // value is double or nothing
              case 2: try
                      {
                        base= (new Double((String)st.nextElement())).doubleValue(); 
                      }
                      catch(NumberFormatException nfe)
                      {
                        DebugMessage("SET_SCALED_LOGARITHMIC: argument specified as base cannot be parsed to double! Argument ignored");
                      }
                      break;
              default: break;
              }
           } 

          // set axis values
          if ("Y".equalsIgnoreCase(axis))
          {
            if ("FALSE".equalsIgnoreCase(enable)) 
            {
              DebugMessage("SET_SCALED_LOGARITHMIC: disabling logarithmic scale for Y axis");
              m_graph.getY1Axis().setScaledLogarithmic(false);
            } else 
            {
              DebugMessage("SET_SCALED_LOGARITHMIC: enabling logarithmic scale for Y axis");
              m_graph.getY1Axis().setScaledLogarithmic(true);
              if (base != 0)
              {
                DebugMessage("SET_SCALED_LOGARITHMIC: setting base to "+base);
                m_graph.getY1Axis().setLogarithmicBase(base);
              }
            }
          }
          else if ("X".equalsIgnoreCase(axis))
          {
            if ("FALSE".equalsIgnoreCase(enable)) 
            {
              DebugMessage("SET_SCALED_LOGARITHMIC: disabling logarithmic scale for X axis");
              m_graph.getX1Axis().setScaledLogarithmic(false);
            } else 
            {
              DebugMessage("SET_SCALED_LOGARITHMIC: enabling logarithmic scale for X axis");
              m_graph.getX1Axis().setScaledLogarithmic(true);
              if (base != 0)
              {
                DebugMessage("SET_SCALED_LOGARITHMIC: setting base to "+base);
                m_graph.getX1Axis().setLogarithmicBase(base);
              }
            }
          }
          else 
          {
           DebugMessage("SET_SCALED_LOGARITHMIC: not a valid Axis identifier!");
          }
          
         } else 
         {
          DebugMessage("SET_SCALED_LOGARITHMIC: no arguments passed with call to set property. Statement ignored!");
         }
         return true;
       }

       /*====================
        * SET_GRAPH_BASELINE
        * ===================*/
        else if (_ID == pSetBaseline)
        {
          if (_object!= null)
          {
            double baseLineVal=0;
            DebugMessage("SET_GRAPH_BASELINE: Trying to set base line val to "+(String)_object);
            try 
            {
              baseLineVal = (new Double((String) _object)).doubleValue();
              m_graph.getY1BaseLine().setValue(baseLineVal);
              DebugMessage("SET_GRAPH_BASELINE: Baseline set to (double) "+baseLineVal);
            } catch (NumberFormatException nfe) 
            {
              DebugMessage("SET_GRAPH_BASELINE: argument passed as a base line value is not a valid number format!");
              return true;          
            }
            
          }
          else
          {
            DebugMessage("SET_GRAPH_BASELINE: No base line value passed - statement ignored");
          }
          return true;
        }
      /*=====================
       * GET DELIMITER
       ======================*/
      else if (_ID == pGetDelimiter)
      {
          // raise custom event to pass the current delimiter to Forms
          dispatchDelimiter();
      }
       
      else 
        {
          return super.setProperty(_ID, _object);
        }
    } 
    catch (Exception ex) 
     {
      ex.printStackTrace();
      return true;
    } 

    finally 
    {
      // do something here
    }
    //never reached but required for the syntax check
    return super.setProperty(_ID, _object);
  }
  
   /** getInternaleGraphType retrieves the int value contained in the attribute passed with 
    *  set_custom_property(), defining the Graph type. If no integer value is found, then the
    *  existing value stored in mGraphType is returned.
    */
  
   private int getInternalGraphType(Object type)
   {
     int iGraphType = 9999999;
     try 
     {
      // call graphTypeAnalyzer.analyze() to determine the int representation of the graph.
      // 0 is returned if no matching BI Graph is registerd for Forms
      iGraphType=graphTypeRegistry.getTypeForString((String) type);
      
      DebugMessage("getInternalGraphType(): type returned for "+type+" = "+iGraphType);
      
      if (iGraphType==9999999) 
      {
        iGraphType = mGraphType;
      }
     } catch (Exception ex) 
     {
        ex.printStackTrace();
        // set default
        iGraphType = mGraphType;
        
     } finally 
     {
     }
     return iGraphType;      
   }

   protected void DebugMessage(String dm)
   {
     //if debug enabled then print message to system out
     if (bDebugMode) 
     {
       System.out.println(debugPrefix+" "+dm);  
     }
   }

   protected void dispatchMouseAction(String msg)
   {
     try{

      CustomEvent ce = new CustomEvent(mHandler,eGraphAction);
      mHandler.setProperty(pGraphInfo,msg);
      dispatchCustomEvent(ce);
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
   }

   public void setDelimiter(String delm)
   {
     sDelimiter=delm;
   }

  /**
   * getDelimiter returns the actual string used to separate values passed in a string. There is a
   * need, e.g. for color values passed for a title string, to change the  delimiter.
   * @return String
   */
   public String getDelimiter ()
   {
     return sDelimiter;
   }

  private void dispatchDelimiter()
   {
     try{

      CustomEvent dde = new CustomEvent(mHandler,eGetDelimiter);
      mHandler.setProperty(pDelimiterInfo,sDelimiter);
      dispatchCustomEvent(dde);
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
   }

  /**
   * method dispatching the number of columns shown in a graph and by raising 
   * a custom event.
  */
   private void returnSeriesCount(int ret)
   {
     try{

      CustomEvent sce = new CustomEvent(mHandler,eGetSeriesCount);
      DebugMessage("returnSeriesCount(): returning "+ret);
      mHandler.setProperty(pGraphInfo,Integer.toString(ret));
      dispatchCustomEvent(sce);
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
   }

  /**
   * public Object[] getTitleFromString(String in) takes a delimited string and creates 
   * a title string, a Font value and a Color value of it. The delimiter used is the 
   * defined delimiter stored in sDelimiter, or ',' as the default. The String syntax is 
   * '<Title>,<color>,<[b][i]>,<size>,<Font Name>'
   * Values can be omitted from right to left, e.g
   * '<Title>,<color>,<[b][i]>,<size>' or
   * '<Title>,<color>,<[b][i]>'
   * 
   */
   public Object[] getTitleFromString(String in)
   {

      Object[] fontProperties    = new Object[4];
      Color fontCol = Color.black;
      String title="";
      String fontName = "Arial";
      int size    = 11;
      int style   = 0;

      // remove empty tokens in beginning and end
      in = handleTokenNullvaluesInStartAndEnd(in);
      
      StringTokenizer tokens = new StringTokenizer(in,sDelimiter);
      int tokenCount = tokens.countTokens();
      DebugMessage("getTitleFromString() received "+tokenCount+" tokens");
      for (int i = 0; i < tokenCount ; i++) 
         {
         switch(i)
          {
            // Title string
            case 0:
              title=(String)tokens.nextElement();
              DebugMessage("getFontFromString(): Title text ="+title);
              break;
           // Font color
            case 1:
              fontCol = colorCodeRegistry.getColorCode((String)tokens.nextElement());
              fontCol= (fontCol!=null?fontCol:Color.black);
              DebugMessage("getFontFromString(): Color ="+fontCol.toString());
              break;
           // Style
            case 2:
              String s = (String) tokens.nextElement();
              DebugMessage("getFontFromString(): Style ="+s);
              if((s.indexOf("n"))>=0)
              {
                style=Font.PLAIN;
              }
              if((s.indexOf("b"))>=0)
              {
                style=Font.BOLD;
              }
              if ((s.indexOf("i"))>=0)
              {
                // bit wise concatenation of italic
                style|=Font.ITALIC;
              }
              break;
           // size
            case 3:
              try
              {
                size = (new Double((String)tokens.nextElement())).intValue();
                DebugMessage("getFontFromString(): Font size ="+size);
              }
              catch(NumberFormatException nfe)
              {
                DebugMessage("getTitleFromString(): "+size+" is an unknow font size and cannot be casted to Integer");
              }
              finally
              {
                break;
              }

           // color
            case 4:
              fontName=(String) tokens.nextElement();
              DebugMessage("getFontFromString(): Font name ="+fontName);
              break;
            default: // ignore
          }
        } 
    fontProperties[0] = title;
    fontProperties[1] = new Font (fontName,style,size);
    fontProperties[2] = fontCol;
    return fontProperties;
   }

  private String handleTokenNullvaluesInStartAndEnd(String in)
  {

    /*
     * tokenizers don't work well if nothing is provided between delimiters. Therefore 
     * all String must be ckecked for this
    */
       
      // check if token is first character
    in = (in.startsWith(sDelimiter) ? " " + in : in);
    
   //check if delimiter is last character in string and if, remove
    while (in.lastIndexOf(sDelimiter) == ((in.length() - 1) - sDelimiter.length()))
    {
      in = in.substring(0, in.lastIndexOf(sDelimiter) - 1);
    }
    return in;
  }

/**
 * private coid handleZerodDataFound() adds teh user defined response to the Graph Bean that shows
 * when no data was passed to the graph
 */
  private void noGraphDataFoundHandler()
  {
    
    // remove Graph from container
    super.remove(0);
    // Create message panel
    NoDataFoundPanel.setVisible(true);
    NoDataFoundPanel.setText(noDataFoundTxt);
    NoDataFoundPanel.setSize(new Dimension(super.getWidth(),super.getHeight()));
    // Set message panel color to color of graph 
    NoDataFoundPanel.setSelectionColor(m_graph.getBackground());
    NoDataFoundPanel.setRequestFocusEnabled(false);
    NoDataFoundPanel.setVerifyInputWhenFocusTarget(false);
    super.add(NoDataFoundPanel);
    recoverGraphToVBean=true;
    }

  public Object getProperty(ID _ID)
  {
    /**********************
     * GET DELIMITER
     * *******************/
    if(_ID == pGetDelimiter)
    {
      return sDelimiter;
    }
    /************************
     * COLUMNS COUNT
     * *********************/
    else if (_ID==pSeriesCount)
    {
      try{
       // retrieve graph column count for further usage
        DebugMessage("SERIES COUNT requested");
        // if rows shown as series then row number = column number
        if (showGraphAsSeries){
          return new Integer(m_graph.getRowCount());
        }
        else
        {
          return new Integer(m_graph.getColumnCount());
        }
      }
      catch (Exception e)
      {
        DebugMessage("====SERIES COUNT Error====");
        DebugMessage(e.getMessage());
        DebugMessage("==========================");
        DebugMessage("returning 0");
        // show 0 if no column can be retrieved
        return new Integer(0);
      }
    }
    else{
      return super.getProperty(_ID);
    }
  }
}