package oracle.forms.demos.enhancedItems2;
// Copyright (c) 2002 Oracle Corp

import oracle.forms.properties.ID;
import oracle.ewt.lwAWT.lwText.LWCommonText;
import java.awt.Point;

/**
 * <H3><FONT COLOR="#336699">Utility class for CursorPos PJCs for Oracle Forms</FONT></H3>
 * This class contains a few common static constants and methods.  Importantly,
 * it also stores the common cursor position or selection.  If an individual
 * instance of the CursorPos PJC is set to use whatever Common is set to, it
 * will come from this class.
 * <br>
 * For more PJC Demos and White Papers see <a href="http://otn.oracle.com/products/forms" target="_blank">Forms on OTN</a>
 *
 * @version 9.0 01/Oct/2002<br>
 *
 * @author Robin Zimmermann
 */

public class CursorPosCommon
{

  /////////////////////////////////////////////////////////////////////////////
  //
  // Variables and Constants
  //
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on for a
   * specific instance of the CursorPos PJC.
   * Used with SET_CUSTOM_PROPERTY() from the Form.
   */
  protected static final ID DEBUGMESSAGES = ID.registerProperty("DEBUGMESSAGES");

  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on for ALL
   * instances of the CursorPos PJC.
   * This will generate a <u>LOT</u> of messages.
   * Used with SET_CUSTOM_PROPERTY() from the Form.
   */
  protected static final ID DEBUGMESSAGES_ALL = ID.registerProperty("DEBUGMESSAGES_ALL");

  /**
   * Property used to set the Caret Position for all instances of the PJC,
   * unless it is overridden by a specific instance.
   * Used with SET_CUSTOM_PROPERTY() from the Form.
   */
  protected final static ID COMMON_CARET_POSITION  = ID.registerProperty("COMMON_CARET_POSITION");

  /**
   * Property used to set the Range Selection for all instances of the PJC,
   * unless it is overridden by a specific instance.
   * Used with SET_CUSTOM_PROPERTY() from the Form.
   */
  protected final static ID COMMON_RANGE_SELECTION = ID.registerProperty("COMMON_RANGE_SELECTION");

  /**
   * Property used to set the Caret Position for a specific instance of the PJC.
   * Used with SET_CUSTOM_PROPERTY() from the Form.
   */
  protected final static ID CARET_POSITION         = ID.registerProperty("CARET_POSITION");

  /**
   * Property used to set the Range Selection for a specific instance of the PJC.
   * Used with SET_CUSTOM_PROPERTY() from the Form.
   */
  protected final static ID RANGE_SELECTION        = ID.registerProperty("RANGE_SELECTION");

  /**
   * Specify that an instance of the CursorPos PJC should use the common
   * setting for it's cursor position or range selection.
   */
  protected static final int COMMON              = -1;

  /**
   * Specify that the field should not control the cursor or range selection,
   * but let Forms do what it would normally do.  This setting can apply to
   * the common position or selection, or to a specific instance of the
   * CursorPos PJC.
   */
  protected static final int FORMS_DEFAULT       = -2;

  /**
   * Specify that the cursor position, or the start or end of the range
   * selection, should be set to the start of the field.  This setting can
   * apply to the common position or selection, or to a specific instance of the
   * CursorPos PJC.
   */
  protected static final int FIELD_START         = -3;


  /**
   * Specify that the cursor position, or the start or end of the range
   * selection, should be set to the end of the field.  This setting can
   * apply to the common position or selection, or to a specific instance of the
   * CursorPos PJC.
   */
  protected static final int FIELD_END           = -4;

  /**
   * Holds the common cursor position or range selection.  If the PJC is
   * setting the cursor position, then that is represented by setting the X
   * and Y variables to the same value.<br>
   * By default, the cursor position is set to the end of the field.
   */
  private static Point CommonCursorSelection = new Point(CursorPosCommon.FIELD_START, CursorPosCommon.FIELD_START);

  /////////////////////////////////////////////////////////////////////////////
  //
  // Getters and Setters
  //
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Get the common range selection.
   *
   * @return a point whose x and y values represent the start and end point
   *         of the selection.
   */
  protected static Point getCommonCursorSelection()
  {
    return CommonCursorSelection;
  } // getCommonCursorSelection()

  /**
   * Set the common range selection.
   *
   * @param pSelection A point whose x and y values represent the start and
   *                   end point of the selection.
   */
  protected static void setCommonCursorSelection(Point pSelection)
  {
    CommonCursorSelection = pSelection;
  } // setCommonCursorSelection()

  /////////////////////////////////////////////////////////////////////////////

  /**
   * Get the start position of the common range selection.
   *
   * @return the start position of the common range selection.
   */
  protected static int getCommonCursorSelectionStart()
  {
    return CommonCursorSelection.x;
  } // getCommonCursorSelectionStart()

  /////////////////////////////////////////////////////////////////////////////

  /**
   * Set the start point of common range selection.
   *
   * @param pPos The start position.
   */
  protected static void setCommonCursorSelectionStart(int pPos)
  {
    CommonCursorSelection.x = pPos;
  } // setCommonCursorSelectionStart()

  /////////////////////////////////////////////////////////////////////////////

  /**
   * Get the end position of the common range selection.
   *
   * @return the end position of the common range selection.
   */
  protected static int getCommonCursorSelectionEnd()
  {
    return CommonCursorSelection.y;
  } // getCommonCursorSelectionEnd()

  /////////////////////////////////////////////////////////////////////////////

  /**
   * Set the end point of common range selection.
   *
   * @param pPos The end position.
   */
  protected static void setCommonCursorSelectionEnd(int pPos)
  {
    CommonCursorSelection.y = pPos;
  } // setCommonCursorSelectionEnd()

  /////////////////////////////////////////////////////////////////////////////
  //
  // Public Methods
  //
  /////////////////////////////////////////////////////////////////////////////

  /**
   * For a given selection, return the start position.
   *
   * @param  pSelection The selection.
   * @return the start position of pSelection.
   */
  public static int getSelectionStart(Point pSelection)
  {
    return pSelection.x;
  } // getSelectionStart()

  /////////////////////////////////////////////////////////////////////////////

  /**
   * Converts the value of the position to a string value.  This is useful
   * for logging output.  Rather than saying, for example, "Setting position
   * to -3", it can say "Setting position to FIELD_START".
   * pPosition may also be a user-defined position, which means it is a number
   * greater than 0.  So be prepared to encode that too.
   *
   * @param  pPosition The position to encode.
   * @return the encoded version of pPosition.
   */
  public static String encodeCaretPosition(int pPosition)
  {
    switch (pPosition)
    {
      case COMMON:
        return "COMMON";
      case FORMS_DEFAULT:
        return "FORMS_DEFAULT";
      case FIELD_START:
        return "FIELD_START";
      case FIELD_END:
        return "FIELD_END";
      default:
        return Integer.toString(pPosition);
    }
  } // encodeCaretPosition

  /////////////////////////////////////////////////////////////////////////////

  /**
   * Converts the selection to a string value.  This is used for communicating
   * the PJC and Forms.  The PJC uses a Point to represent the selection, but
   * there is no equivalent object in PL/SQL, so a selection is represented
   * by a text string.  e.g. "-3 5" which equals from FIELD_START to position
   * 5.
   *
   * @param  pSelection The selection to encode.
   * @return the encoded version of pSelection.
   */
  public static String encodeSelection(Point pSelection)
  {
    return pSelection.x + " " + pSelection.y;
  } // encodeSelection()

  /////////////////////////////////////////////////////////////////////////////

  /**
   * The opposite of encodeSelection.  It takes a string represenation of a
   * selection and converts it to a Point.
   *
   * @param  pSelection The selection to decode.
   * @return pSelection as a Point.
   */
  public static Point decodeSelection(String pSelection)
  {
    Integer x;
    Integer y;
    int     vSeperator = pSelection.indexOf(' ');

    try
    {
      x = new Integer(pSelection.substring(0, vSeperator));
      y = new Integer(pSelection.substring(vSeperator + 1));
    }
    catch ( NumberFormatException nfe )
    {
      return null;
    }
    return new Point(x.intValue(), y.intValue());
  } // decodeSelection()

  /////////////////////////////////////////////////////////////////////////////
  //
  // Protected Methods
  //
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Converts a string value containing TRUE or FALSE
   * as a string to the equivalent boolean.
   *
   * @param value String containing the words True or false in any case or the Forms Constants PROPERTY_TRUE/PROPERTY_FALSE
   * @return boolean value of the value
   */
  protected final static boolean toBoolean(Object value, boolean defaultValue)
  {
     boolean RC = defaultValue;
     if (value.toString().equalsIgnoreCase("false")||value.toString().equals("5"))
        RC = false;
     else if (value.toString().equalsIgnoreCase("true")||value.toString().equals("4"))
        RC = true;
     return RC;
  } // toBoolean()

  /////////////////////////////////////////////////////////////////////////////

  /**
   * This method returns the action for this field.  It determines if the common
   * setting is used, or if it is has been overridden.<br>
   * If the start or end point is set to COMMON, then the position from the
   * common values are used.<br>
   * If the start or end point is FIELD_START, then the position is set to 0.<br>
   * If the start or end point in FIELD_END, then the position is set to the
   * length of the text in the field.
   *
   *
   * @param  pCursorSelection The cursor selection for the given field.
   * @param  pComp The text field that the PJC is acting on.
   * @return a new Point which has resolved all constants to a value.
   */
  protected static Point resolveSelectionForThisField(Point pCursorSelection, LWCommonText pComp)
  {

    Point vSelection = new Point(0, 0);

    //
    // First determine if we should use the selection that was passed in, or
    // use Common.
    //
    if (pCursorSelection.x == CursorPosCommon.COMMON)
    {
      vSelection.x = CursorPosCommon.getCommonCursorSelectionStart();
//      vSelection.y = CursorPosCommon.getCommonCursorSelectionEnd();
    }
    else
    {
      vSelection.x = pCursorSelection.x;
//      vSelection.y = pCursorSelection.y;
    }
    if (pCursorSelection.y == CursorPosCommon.COMMON)
    {
//      vSelection.x = CursorPosCommon.getCommonCursorSelectionStart();
      vSelection.y = CursorPosCommon.getCommonCursorSelectionEnd();
    }
    else
    {
//      vSelection.x = pCursorSelection.x;
      vSelection.y = pCursorSelection.y;
    }

    //
    // Now resolved any constants to their correct values.
    //

    if (vSelection.x == CursorPosCommon.FIELD_START)
    {
      vSelection.x = 0;
    }
    else if (vSelection.x == CursorPosCommon.FIELD_END)
    {
      vSelection.x = pComp.getTextLength();
    }

    if (vSelection.y == CursorPosCommon.FIELD_START)
    {
      vSelection.y = 0;
    }
    else if (vSelection.y == CursorPosCommon.FIELD_END)
    {
      vSelection.y = pComp.getTextLength();
    }

    //
    // Note that if the X or Y values are FORMS_DEFAULT, this method doesn't
    // do anything.  It assumes the calling method will handle that case.
    //

    return vSelection;

  } // resolveSelectionForThisField()

} // CursorPosCommon
