package oracle.forms.demos.enhancedItems2;
// Copyright (c) 2002 Oracle Corp

import oracle.forms.ui.*;
import oracle.forms.properties.ID;
import oracle.forms.engine.*;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.Point;
import oracle.ewt.lwAWT.lwText.LWCommonText;

/**
 * <H3><FONT COLOR="#336699">CursorPos Pluggable Java Component for Oracle Forms</FONT></H3>
 * This class is a Pluggable Java Component for Oracle Forms, that implements
 * a method for controlling the position of the cursor or range selection in a
 * text area
 *
 * <br><br>
 * <b>Cursor Position</b><br>
 * This class lets you specify the cursor position as either FIELD_START, 
 * FIELD_END, or a user-defined position (which should be greater than or equal
 * to 0).  You can also specify COMMON as a position, which means this instance
 * will use whatever Common has been set to.
 *
 * <br><br>
 * <b>Range Selection</b><br>
 * You can also specify a range selection that will highlight text.  You set the
 * start and end positions.  The valid values are the same as for the Cursor
 * Position.
 *
 * <br><br>
 * For more PJC Demos and White Papers see <a href="http://otn.oracle.com/products/forms" target="_blank">Forms on OTN</a>
 *
 * @version 9.0 01/Oct/2002<br>
 *
 * @author Robin Zimmermann
 */
public class CursorPosTextArea extends VTextArea
{

  /////////////////////////////////////////////////////////////////////////////
  //
  // Variables and Constants
  //
  /////////////////////////////////////////////////////////////////////////////

  /**
   * The classname used for debugging purposes (in the log method).
   * use getClass().getName(); if you want the
   * full package name
   */
  private final String CLASSNAME = this.getDefaultName();

  /**
   * Holds the cursor position or range selection for this instance.  If the PJC
   * is setting the cursor position, then that is represented by setting the X
   * and Y variables to the same value.<br>
   * By default, this cursor selection is set to common, so will use whatever
   * common is set to, unless overridden.
   */
  private Point CursorSelection = new Point(CursorPosCommon.COMMON, CursorPosCommon.COMMON);

  /**
   * Do we want to debug for this class?  Set this to true to see debug messages.<br>
   * This can be set at any time use SET_CUSTOM_PROPERTY() from the Form.
   */
  private boolean Debug = false;

  /**
   * Do we want to debug for all instances of this class?  Set to true to see
   * debug messages for all instances of the this PJC.<br>
   * This can be set at any time use SET_CUSTOM_PROPERTY() from the Form.
   */
  private static boolean DebugAll = false;

  private LWCommonText mTextArea = null;

  /////////////////////////////////////////////////////////////////////////////
  //
  // Constructors
  //
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Constructor for the CursorPosTextField PJC
   */
  public CursorPosTextArea()
  {
    super();
  } // CursorPosTextArea()

  /////////////////////////////////////////////////////////////////////////////
  //
  // Public Methods
  //
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Implementation of IView interface. This code in the PJC is called when
   * PL/SQL in Forms issues a GET_CUSTOM_PROPERTY call The value returned from
   * this method is returned to PL/SQL
   *
   * @param pId property ID
   * @return    value of the property
   * @see IView
   */
  public Object getProperty(ID pId)
  {
    if (pId == CursorPosCommon.CARET_POSITION)
    {
      log("getProperty(" + pId.getName() + ")");
      return "" + CursorPosCommon.getSelectionStart(CursorSelection);
    }
    else if (pId == CursorPosCommon.COMMON_CARET_POSITION)
    {
      log("getProperty(" + pId.getName() + ")");
      return "" + CursorPosCommon.getSelectionStart(CursorPosCommon.getCommonCursorSelection());
    }
    if (pId == CursorPosCommon.RANGE_SELECTION)
    {
      log("getProperty(" + pId.getName() + ")");
      return CursorPosCommon.encodeSelection(CursorSelection);
    }
    else if (pId == CursorPosCommon.COMMON_RANGE_SELECTION)
    {
      log("getProperty(" + pId.getName() + ")");
      return CursorPosCommon.encodeSelection(CursorPosCommon.getCommonCursorSelection());
    }
    else if (pId == CursorPosCommon.DEBUGMESSAGES)
    {
      log("getProperty(" + pId.getName() + ")");
      return "" + Debug;
    }
    else if (pId == CursorPosCommon.DEBUGMESSAGES_ALL)
    {
      log("getProperty(" + pId.getName() + ")");
      return "" + DebugAll;
    }
    else
    {
      return super.getProperty(pId);
    }
  } // getProperty()

  /////////////////////////////////////////////////////////////////////////////

  /**
   * Implementation of IView interface. This code in the PJC is called when
   * PL/SQL in Forms issues a SET_CUSTOM_PROPERTY call
   *
   * @param pId    property to be set
   * @param vValue new value of the property
   * @return       true (if the property could be set), false otherwise
   * @see IView
   */
  public boolean setProperty(ID pId, Object pValue)
  {

    if (mTextArea == null)
    {
      // VTextArea itself is a ScrollBox containing a text area 
      // get the actual Text component
      mTextArea = (LWCommonText)this.getComponent(0);
    }

    if (pId == CursorPosCommon.CARET_POSITION)
    {
      //
      // A cursor position really uses the cursor selection, but sets the
      // start and end to be the same values.
      //
      int vValue = Integer.parseInt(pValue.toString());
      CursorSelection.x = vValue;
      CursorSelection.y = vValue;
      log(pId.getName() + ": " + CursorPosCommon.encodeCaretPosition(vValue));
      return true;
    }
    else if (pId == CursorPosCommon.COMMON_CARET_POSITION)
    {
      //
      // A cursor position really uses the cursor selection, but sets the
      // start and end to be the same values.
      //
      int vValue = Integer.parseInt(pValue.toString());
      CursorPosCommon.setCommonCursorSelectionStart(vValue);
      CursorPosCommon.setCommonCursorSelectionEnd(vValue);
      log(pId.getName() + ": " + CursorPosCommon.encodeCaretPosition(vValue));
      return true;
    }
    if (pId == CursorPosCommon.RANGE_SELECTION)
    {
      Point vValue = CursorPosCommon.decodeSelection(pValue.toString());
      CursorSelection.x = vValue.x;
      CursorSelection.y = vValue.y;
      log(pId.getName() + ": " + CursorPosCommon.encodeCaretPosition(vValue.x)+" "+CursorPosCommon.encodeCaretPosition(vValue.y));
      return true;
    }
    else if (pId == CursorPosCommon.COMMON_RANGE_SELECTION)
    {
      Point vValue = CursorPosCommon.decodeSelection(pValue.toString());
      CursorPosCommon.setCommonCursorSelectionStart(vValue.x);
      CursorPosCommon.setCommonCursorSelectionEnd(vValue.y);
      log(pId.getName() + ": " + CursorPosCommon.encodeCaretPosition(vValue.x)+" "+CursorPosCommon.encodeCaretPosition(vValue.y));
      return true;
    }
    else if (pId == CursorPosCommon.DEBUGMESSAGES)
    {
      Debug = CursorPosCommon.toBoolean(pValue,false);
      boolean vDebug = Debug;
      Debug = true;
      log(pId.getName() + ": " + vDebug);
      Debug = vDebug;
      return true;
    }
    else if (pId == CursorPosCommon.DEBUGMESSAGES_ALL)
    {
      DebugAll = CursorPosCommon.toBoolean(pValue,false);
      boolean vDebug = Debug;
      Debug = true;
      log(pId.getName() + ": " + DebugAll);
      Debug = vDebug;
      return true;
    }
    else if (pId == ID.SELECTION)
    {
      //
      // Here is the clever part.  The SELECTION property is not to be set
      // explicitly from Forms in the same way as, say, CARET_POSITION.  It is
      // controlled internally by Forms.
      // What we are doing is <i>overriding</i> the way Forms behaves.  When
      // Forms is ready to put the cursor in a field, it calls this property.
      // By default, this property will select the whole field, or put the
      // cursor in a certain position if the Keep Cursor Position property is
      // true.
      // We are intercepting this call.  If our instance is not using the Forms
      // default, then we will set the cursor position to where we want.
      // Clever, eh?
      //

      Point vSelection = CursorPosCommon.resolveSelectionForThisField(CursorSelection, mTextArea);

      //
      // Are we doing the Forms default, or are we doing something different?
      // Note that we don't explicitly move the cursor ourselves.  We call the
      // super method to do the work.  This is true whether we use the Forms
      // default or not.
      //
      if (vSelection.x == CursorPosCommon.FORMS_DEFAULT)
      {
        log(pId.getName() + ": FORMS_DEFAULT");
        return super.setProperty(pId, pValue);
      }
      else
      {
        log(pId.getName() + ": " + vSelection.x+" "+vSelection.y);
        return super.setProperty(pId, vSelection);
      }

    }
    else
    {
      //log("[INTERNAL] "+pId.getName());  // This generates a LOT of output, so comment out when not needed.
      return super.setProperty(pId, pValue);
    }
  } // setProperty()

  /////////////////////////////////////////////////////////////////////////////
  //
  // Protected Methods
  //
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Utility function to print out a debug message to the Java Console.
   * @param pMsg String to display, this will be prefixed with the classname of the PJC.
   */
  private final void log(String pMsg)
  {
    if(Debug||DebugAll)
        System.out.println(CLASSNAME + ": " + pMsg);
  } // log()

  /////////////////////////////////////////////////////////////////////////////
  //
  // Private Methods
  //
  /////////////////////////////////////////////////////////////////////////////

} // CursorPosTextArea
