/* Copyright (c) Oracle Corporation 2000.  All Rights Reserved. */

package oracle.forms.demos.beans;

import java.util.*;
import java.net.InetAddress;
import java.text.*;

import oracle.forms.ui.*;
import oracle.forms.properties.*;
import oracle.forms.handler.*;

/**
 * <H3><FONT COLOR="003399">Oracle Forms JavaBean / PJC Example</FONT></H3>
 * <P>This JavaBean fetches information from the local client system and
 * delivers it to Forms when the <code>get_custom_property</code> builtin is invoked.</P>
 * <P>The class will need to be signed
 * before being used in general purpose Forms applications since access to the System properties
 * is an activity that is defined outside of the default Java sandbox.</P>
 * Version 2 from Forms 9<i>i</i> can be used either as a PJC or as dynamically accessed JavaBean
 * The ability to get the Java version and also any arbitary system property has also been added
 *
 * @version 2.0 05/Nov/2001
 * @author  Steve Button, Duncan Mills

 */

 public class GetClientInfo
       extends VBean
{
  /**
   * The CLASSNAME for this class, used in debugging
   */
  private final String CLASSNAME = getClass().getName();

  /**
   * The default message to return for a null localhost value
   */
  private final String DEFAULTMESSAGE = "NOT AVAILABLE";

  /**
   * Register a custom property, used to invoke the getUsername() JavaBean method
   */
  static final ID USERNAME = ID.registerProperty("USERNAME");
  /**
   * Register a custom property, used to invoke the getHostname() JavaBean method
   */
  static final ID HOSTNAME = ID.registerProperty("HOSTNAME");
  /**
   * Register a custom property, used to invoke the getIPAddress() JavaBean method
   */
  static final ID IPADDRESS = ID.registerProperty("IPADDRESS");
  /**
   * Register a custom property, used to invoke the getClientDate() JavaBean method
   */
  static final ID CLIENTDATE = ID.registerProperty("CLIENTDATE");

  /**
   * New for version 2.0 Register a custom property to get the client Java version
   */
  static final ID CLIENT_JAVA_VERSION = ID.registerProperty("CLIENT_JAVA_VERSION");


  /**
   * Are we debugging?
   */
  private boolean m_debug = false;

  /**
   * The InetAddress object used to store the localhost information
   */
  private InetAddress m_localhost = null;

  /**
   * Stores the handler for this view class
   */
  IHandler m_handler = null;


  /**
   * The constructor for the GetUsername JavaBean class
   */
  public void GetClientInfo()
  {
    log("constructing GetClientInfo object");
  }

  /**
   * Method in the IView interface that is called when the component is first
   * initialised from by the Forms Server.
   *
   * @param  a reference to the Handler object for this class
   */
  public void init(IHandler handler)
  {
    log("init()");
    m_handler = handler;
    findLocalHost();
  }

  /**
   * Method in the IView interface that is called when the Forms Server wishes to
   * obtain the value of a specific property from the class.
   *
   * @param pid the property to be returned
   * @return the value of the property that was requested
   */
  public Object getProperty(ID pid)
  {
    log("getProperty(" + getPropertyName(pid) + ")");

    if(pid == USERNAME)
    {
      return (getUsername());
    }
    else if(pid == HOSTNAME)
    {
      return(getHostname());
    }
    else if(pid == IPADDRESS)
    {
      return(getIPAddress());
    }
    else if(pid == CLIENTDATE)
    {
      return(getDate());
    }
    else if(pid == CLIENT_JAVA_VERSION)
    {
      return(getSystemProperty("java.version"));
    }
    // let the VBean superclass handle the other properties
    else
      return(super.getProperty(pid));
  }

  /**
   * Find the localhost InetAddress for the client machine in which this
   * class is currently running.  This method must have been called before
   * the other convenience functions are called since the depend on the
   * value it provides.
   */
  private void findLocalHost()
  {
    if(m_localhost == null)
    {
      try
      {
        log("obtaining LocalHost info from InetAddress");
        m_localhost = InetAddress.getLocalHost();
      }
      catch(Exception e)
      {
        log("ERROR obtaining LocalHost info from InetAddress");
        e.printStackTrace();
      }
    }
  }


  /**
   * Obtains the username from the client's JVM System properties with "user.name"
   * @return the client username
   */
  public String getUsername()
  {
     return System.getProperty("user.name");
  }
  /**
   * Obtains the named property from the client's JVM System properties
   * @return the property value
   */
  public String getSystemProperty(String systemProperty)
  {
    log("getSystemProperty(" + systemProperty + ")");
    return System.getProperty(systemProperty);
  }

  /**
   * Obtains the hostname of the local client machine
   *
   * @return the hostname or the default message in case of error
   */
  public String getHostname()
  {
    findLocalHost();
    if(m_localhost != null)
      return m_localhost.getHostName();
    else
      return DEFAULTMESSAGE;
  }

  /**
   * Obtains the IP address of the local client machine
   *
   * @ return the IP Address or the default message in case of error
   */
  public String getIPAddress()
  {
    findLocalHost();
    if(m_localhost != null)
      return m_localhost.getHostAddress();
    else
      return DEFAULTMESSAGE;
  }

  /**
   * A convenience function to use the ID.getName() method to get the text version of this pid
   * @param pid the property to display the name of
   * @return the property name
   */
  private String getPropertyName(ID pid)
  {
    try
    {
      return pid.getName();
    }
    catch(Exception e)
    {
    }
    return null;
  }

  /**
   * Obtain the date off of the local client machine.
   *
   * @return the current date from the client machine
   */
  public String getDate()
  {
    DateFormat format = DateFormat.getDateInstance(DateFormat.FULL);
    format.setCalendar(Calendar.getInstance());
    return format.format(format.getCalendar().getTime());
  }

  /**
   * Sets debug mode
   * @param should debugging to console be switched on?
   */
  public void setDebug(boolean debugMode)
  {
    log("setDebug(" + debugMode + ")");
    m_debug = debugMode;
  }

  /**
   * @return is debug mode on or off?
   */
  public boolean getDebug()
  {
    log("getDebug()");
    return m_debug;
  }


  /**
   * Convenience method to display an message out to the console.  Examines the
   * m_Debug and m_DebugALL settings to see if it should output the message.  Outputs message in
   * format <classname : msg >
   *
   * @param the message to be output
   */
  public void log(String msg)
  {
    if(m_debug)
        System.out.println(CLASSNAME + ": " + msg);
  }

  /**
   * The test method which allows class to be run from the command line to determine
   * if it is functioning correctly.
   *
   * @param args[] the command line arguments
   */
  public static void main(String args[])
  {

    System.out.println("Running GetClientInfo()");

    GetClientInfo ci = new GetClientInfo();

    Properties props = System.getProperties();
    Enumeration e = props.propertyNames();
    while(e.hasMoreElements())
    {
      String key = (String)e.nextElement();
      System.out.println(key + " = " + props.getProperty(key));
    }
    // call the findLocalHost function
    ci.findLocalHost();
    System.out.println("Username : " + ci.getUsername());
    System.out.println("Hostname : " + ci.getHostname());
    System.out.println("IP       : " + ci.getIPAddress());
    System.out.println("Date     : " + ci.getDate());
    System.exit(1);
  }

}
