package oracle.forms.demos.beans;

import oracle.ewt.lwAWT.LWLabel;
import java.awt.Color;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.AWTEventMulticaster;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.util.EventListener;
import java.awt.Cursor;


/**
 *
 * <H3><FONT COLOR="003399">Oracle Forms JavaBean Example</FONT></H3>
 *
 * A Hyperlink is a JavaBean that mimics the look and actions of a standard
 * web browser hyperlink.  It can be used in Java applications to provide a
 * similar working experience to that obtained from HTML based applications.
 *
 * The hyperlink supports the following states:
 *
 * <TABLE BORDER="1">
 * <TR>
 * <TD><code>normal</code></TD>
 * <TD><i>The default state of the hyperlink when the user has/had no interaction[s]
 * with it.</TD></TR>
 * <TR>
 * <TD><code>active</code></TD>
 * <TD><i>The state of the hyperlink when the user has moved the mouse over it.</TD></TR>
 * <TR>
 * <TD><code>visited</code></TD>
 * <TD><i>The state of the hyperlink when the user clicked on the hyperlink component
 * during this particular application session.  Will expire when the user ends the
 * application.</TD></TR>
 * </TABLE>
 *
 * <P>
 * The hyperlink will not perform any default actions when the user clicks on it.  Any
 * component that is interested in the actions of the hyperlink must register itself
 * as an <code>ActionListener</code> with the hyperlink.  The hyperlink will notify the interested
 * parties when it has been clicked through an <code>ActionEvent</code>
 * </P>
 *
 * @version 9.0  29/Oct/2001
 * @author Steve Button
 * @author Duncan Mills
 */
public class Hyperlink extends LWLabel implements MouseListener
{
  /**
   * the classname used for debugging purposes
   */
  private final String CLASSNAME = this.getClass().getName();

  /**
   * The URL for this hyperlink
   */
  private String m_URL= null;
  /**
   * The color representing the normal Hyperlink.  Default color is blue
   */
  private Color  m_normalColor;
  /**
   * The color representing the active Hyperlink.  Default color is blue.
   */
  private Color m_activeColor;
  /**
   * The color representing the visited Hyperlink.  Default color
   * is red.
   */
  private Color m_visitedColor;
  /**
   * The Color defaults used for this component when it is passed null
   * constructor values.
   */
  private static Color DEFAULTNORMAL  = Color.blue;
  private static Color DEFAULTACTIVE  = Color.blue;
  private static Color DEFAULTVISITED = Color.decode("#840084");

  /**
   * Used to access the AWT Event management facilities
   * to store and fire events on interested ActionEvent listeners.
   */
  private ActionListener m_actionListener = null;

  /**
   * Used to access the AWT Event management facilities
   * to store and fire events on interested MouseEvent listeners.
   */
  private MouseListener m_mouseListener = null;

  /**
   * Represents the state of the hyper link
   */
  boolean m_isClicked = false;
  /**
   * Are we debugging?
   */
  private boolean m_debug = false;
  /**
   * Are we debugging All instances?
   */
  private static boolean m_debugAll = false;


  public Hyperlink()
  {
    this("Empty Hyperlink");
  }

  /**
   * Create a hyperlink with no specific URL target
   *
   * @param label Text of label
   */
  public Hyperlink(String label)
  {
    this(label, null);
  }

    /**
   * Create a hyperlink using the given URL.  Set the normal color
   * to the default.
   *
   * @param label Text of linklabel
   * @param URL   The URL to associate with this linklabel
   */
  public Hyperlink(String label, String URL)
  {
    this(label, URL, DEFAULTNORMAL);
  }

  /**
   * Creates a hyperlink using the given URL with the
   * specified color.  Sets the visited color to the default.
   *
   * @param label Text of label
   * @param URL   Universal Resource Locator (URL)
   * @param color    Color of the normal hyperlink
   */
  public Hyperlink(String label, String URL, Color normalcolor)
  {
    this(label,URL,normalcolor,DEFAULTVISITED);
  }

  /**
   * Creates a hyperlink using the given URL with the
   * specified color.
   *
   * @param label Text of label
   * @param URL   Universal Resource Locator (URL)
   * @param color    Color of normal hyperlink
   * @param color    Color of visited
   */
  public Hyperlink(String label, String URL, Color normalcolor, Color visitedcolor)
  {
    super(label);
    setURL(URL);
    setNormalColor(normalcolor);
    setActiveColor(m_normalColor);
    setVisitedColor(visitedcolor);
    super.addMouseListener(this);
  }

  /**
   * Set the label for the HyperLink.
   *
   * @param Label the label to display
   *
   */
  public void setLabel(String label)
  {
    log("setLabel(" + label + ")");
    m_isClicked=false;
    super.setText(label);
  }

  /**
   * Get the label for the HyperLink.
   *
   * @return The label for this HyperLink
   *
   */
  public String getLabel()
  {
    log("getLabel()");
    m_isClicked=false;
    return super.getText();
  }

  /**
   * Sets the URL to associate with this hyperlink.
   *
   * @param URL   The associated URL.
   */
  public void setURL(String URL)
  {
    log("setURL("+ URL + ")" );
    this.m_URL=URL;
    m_isClicked=false;
   }

  /**
   * Returns the URL associated with this hyperlink.
   *
   * @return The URL assosicated with this hyperlink as a String
   */
  public String getURL()
  {
    log("getURL()");
    return m_URL;
  }

  /**
   * Sets the visited color for this hyperlink.  The visited color will appear
   * when the link has been clicked as determined by the isClicked attribute.
   *
   * @param color Color of visited link
   */
  public void setVisitedColor(Color newVisitedColor)
  {
    log("setVisitedColor(" + newVisitedColor.toString() + ")" );
    m_visitedColor = newVisitedColor;
    repaint();
  }

  /**
   * Gets the visited color for this hyperlink.
   *
   * @return color  Color of visited link
   */
  public Color getVisitedColor()
  {
    log("getVisitedColor()");
    return m_visitedColor;
  }

   /**
    * Sets the normal color for this hyperlink.  The normal color appears is used
    * when the link is first and subsequently drawn, up until the point where
    * it is actively clicked.
    *
    * @return color Color of normal hyperlink
    */
  public void setNormalColor(Color newNormalColor)
  {
    log("setNormalColor(" + newNormalColor.toString() + ")" );
    m_normalColor = newNormalColor;
    setForeground(m_normalColor);
    repaint();
  }

  /**
   * Gets the normal color for this hyperlink.
   *
   * @return color Color of normal hyperlink
   */
  public Color getNormalColor()
  {
    log("getNormalColor()");
    return m_normalColor;
  }

  /**
   * Sets the active color for this hyperlink.  The active color appears when
   * the mouse is moved over the hyperlink to signify that it is a usable
   * component with an associated action.
   *
   * @return color Color of active hyperlink
   */
  public void setActiveColor(Color newActiveColor)
  {
    log("setActiveColor(" + newActiveColor.toString() + ")" );
    m_activeColor = newActiveColor;
    repaint();
  }

  /**
   * Gets the active color for this hyperlink.
   *
   * @return color Color of active hyperlink
   */
  public Color getActiveColor()
  {
    log("getActiveColor()");
    return m_activeColor;
  }

  /**
   * Sets debug mode - allows switch on for all instances
   */
  public void setDebug(boolean debugMode, boolean forAll)
  {
    log("setDebug(" + debugMode + "," + forAll + ")");
    if (forAll)
      m_debugAll = debugMode;
    else
      m_debug = debugMode;
  }

  /**
   * Sets debug mode
   */
  public void setDebug(boolean debugMode)
  {
    log("setDebug(" + debugMode + ")");
    this.setDebug(debugMode,false);
  }

  /**
   * Gets debug mode
   */
  public boolean getDebug()
  {
    log("getDebug()");
    if ((m_debug)||(m_debugAll))
      return true;
    else
      return false;
  }

  /**
   * Paints the hyperlink.
   *
   * @param g    The Graphics object to paint on
   */
  public void paint(Graphics g)
  {
    // let the label itself handle most of the hardwork
    super.paint(g);

    // Check that a hyperlink was specified
    if (m_URL == null) return;

    // Get width of hyperlink label component
    Dimension d = getSize();
    int width = d.width;
    int height= d.height;

    // Get width of URL label to calculate the line position
    FontMetrics metrics = getFontMetrics(getFont());
    int strWidth = metrics.stringWidth(getText());
    int yoffset = (height /2 ) + metrics.getHeight() /2 ;
    int xoffset = width - strWidth ;
    g.drawLine(0, yoffset, strWidth, yoffset);
  }

  //
  // This section of the code handles the ActionListeners registration,
  // removal and event dispatchine
  //

  /**
   * Adds the specified action listener object to receive action events from
   * this hyperlink.
   *
   * @param listener  ActionListener to add to event queue
   */
  public synchronized void addActionListener(ActionListener listener)
  {
    m_actionListener = AWTEventMulticaster.add(m_actionListener, listener);
  }

  /**
   * Removes the specified action listener object from the list of action listeners
   * associated with this hyperlink.
   *
   * @param listener  ActionListener to add to event queue
   */
  public synchronized void removeActionListener(ActionListener listener)
  {
    m_actionListener = AWTEventMulticaster.remove(m_actionListener, listener);
  }

  //
  // This section of the code handles the MouseListeners registration,
  // removal, etc.
  //

  /**
   * Adds the specified action listener object to receive action events from
   * this hyperlink.
   *
   * @param listener  ActionListener to add to event queue
   */
  public synchronized void addMouseListener(MouseListener listener)
  {
    m_mouseListener = AWTEventMulticaster.add(m_mouseListener, listener);
  }

  /**
   * Removes the specified action listener object from the list of action listeners
   * associated with this hyperlink.
   *
   * @param listener  ActionListener to add to event queue
   */
  public synchronized void removeActionListener(MouseListener listener)
  {
    m_mouseListener = AWTEventMulticaster.remove(m_mouseListener, listener);
  }

  //
  // This section of the code provides the method implementations for the
  // specified MouseListener interface
  //

  /**
   * Event handler for when the mouse clicks the hyperlink
   *
   * @param MouseEvent Event
   */
  public void mouseClicked(MouseEvent e)
  {
    m_isClicked=true;
    setForeground(m_visitedColor);
    if (m_actionListener != null)
    {
      String actionString;
      if (m_URL == null)
        actionString = "LINK CLICKED";
      else
        actionString = m_URL;

      ActionEvent actionEvent =
        new ActionEvent(this, ActionEvent.ACTION_PERFORMED,actionString);
      m_actionListener.actionPerformed(actionEvent);
    }
    repaint();
  }

  /**
   * Event handler for when the mouse enters the hyperlink
   *
   * @param MouseEvent Event
   */
  public void mouseEntered(MouseEvent e)
  {
    setCursor (new Cursor(Cursor.HAND_CURSOR));
    if(!m_isClicked)
    {
      setForeground(m_activeColor);
      repaint();
    }

    // propagate the mouse event back to any other mouse listeners
    if(m_mouseListener!=null)
      m_mouseListener.mouseEntered(e);
  }

  /**
   * Event handler for when the mouse leaves the hyperlink
   *
   * @param MouseEvent Event
   */
  public void mouseExited(MouseEvent e)
  {
    setCursor (new Cursor(Cursor.DEFAULT_CURSOR));
    if(!m_isClicked)
    {
      setForeground(m_normalColor);
      repaint();
    }

    // propagate the mouse event back to any other mouse listeners
    if(m_mouseListener!=null)
      m_mouseListener.mouseExited(e);
  }

  /**
   * Not implemented
   */
  public void mousePressed(MouseEvent e)
  {
    // no implementation required
  }

  /**
   * Not implemented
   */
  public void mouseReleased(MouseEvent e)
  {
    // no implementation required
  }

  /**
   * Convenience method to display an message out to the console.  Examines the
   * m_Debug and m_DebugALL settings to see if it should output the message.  Outputs message in
   * format <classname : msg >
   *
   * @param the message to be output
   */
  public void log(String msg)
  {
    if(m_debug||m_debugAll)
        System.out.println(CLASSNAME + ": " + msg);
  }
}