package oracle.forms.demos.enhancedItems;

import java.awt.*;
import java.awt.event.*;
import java.awt.Cursor;

import oracle.forms.ui.VButton;
import oracle.forms.handler.IHandler;
import oracle.forms.properties.ID;
import oracle.forms.properties.Property;
import oracle.ewt.lwAWT.lwText.LWTextArea;
import oracle.ewt.lwAWT.lwText.LWCommonText;


/**
 * <H3>Oracle Forms Server PJC Example</H3>
 *
 * This is a PJC that implements a normal Forms button (Text or Iconic).  When the user moves
 * The mouse over the button, a separate feild is populated with information - much like a Tooltip
 * except that the information appears in another field rather than a popup
 * <code>oracle.forms.ui.VButton</code> is subclassed so only the additional functionality is required
 * to be added.
 * Like the rollover button example this button can also be made rounded by simply adding leading or
 * trailing brackets to the label of the button e.g. '(save)'
 *
 * We register the following new custom properties for the button:
 * <ul>
 * <li><b>INFOBUTTON_FIELDPOS</b> - A comma separated list indicating the X,Y position of the target field in pixels</li>
 * <li><b>INFOBUTTON_TEXT</b>  - The text to display when the button has mouse focus</li>
 * </ul>
 *
 * @version 1.0 03/Nov/2001 created for 9<i>i</i> Demos<br>
 * @author  Duncan Mills
 */
public class InfoButton extends VButton
{
  /**
   * the property registered to specify the position of the Info Field
   */
  public final static ID INFOBUTTON_FIELDPOS        = ID.registerProperty("INFOBUTTON_FIELDPOS");

  /**
   * the property registered to specify the text for the Info Field
   */
  public final static ID INFOBUTTON_TEXT            = ID.registerProperty("INFOBUTTON_TEXT");


  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on
   */
  private static final ID DEBUGMESSAGES             = ID.registerProperty("DEBUGMESSAGES");

  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console for ALL INSTANCES of this PJC
   * so we can see what is going on
   * This will generate a <u>LOT</u> of messages
   */
  private static final ID DEBUGMESSAGES_ALL         = ID.registerProperty("DEBUGMESSAGES_ALL");

  /**
   * the classname used for debugging purposes
   * use getClass().getName(); if you want the
   * full package name
   */
  private final String CLASSNAME = this.getDefaultName();


  /**
   * the handle of the Info Field
   */
  private LWCommonText m_infoField;

  /**
   * is the InfoButton output field located
   */
  private boolean m_isFound = false;


  /**
   * the mouse adaptor that will be used to display and hide the text
   */
  private InfoButtonMouseAdapter m_mouseAdapter;

  /**
   * storage for the text to display
   */
  private String  m_text;

  /**
   * storage for the previous text from the feild
   */
  private String m_savedText;



  /**
   * do we want to debug for this class?  Set this to true to see debug messages.
   */
  private boolean m_debug = false;

  /**
   * do we want to debug for all instances of this class?
   */
  private static boolean m_debugAll = false;


  public InfoButton()
  {
    super();
    log("Debugging on: Creating Button Instance");
  }

  /**
   * Implementation of IView interface which sets a requested property to a given value
   * If the property being set is LABEL then special processing is undertaken.
   * If the Label is prefixed or suffixed with rounded brackets we round that edge of the
   * button in Oracle Look and Feel
   *
   * @param property    property to be set.
   * @param value value of the property id.
   * @return      true if the property could be set, false otherwise.
   * @see oracle.forms.ui.IView
   */
  public boolean setProperty(ID property, Object value)
  {
    if (property == ID.LABEL)
    {
      log("Setting Label to " + value.toString());
      String label = value.toString().trim();

      if (label.startsWith("("))
      {
        log("Rounding left edge of " + label);
        setLeftmost(true);
        label = label.substring(1);
      }
      if (label.endsWith(")"))
      {
        log("Rounding right edge of " + label);
        setRightmost(true);
        label = label.substring(0,label.length()-1);
      }
      return super.setProperty(property, label);
    }
    else if (property == INFOBUTTON_TEXT)
    {
      m_text = value.toString();
      return true;
    }
    else if (property == INFOBUTTON_FIELDPOS)
    {
      log("Setting FieldPos to " + value.toString());
      int i = value.toString().indexOf(",");
      if (i > 0)
      {
        int x = Integer.parseInt(value.toString().substring(0,i));
        int y = Integer.parseInt(value.toString().substring(i+1));
        defineInfoField(new Point(x,y));
      }
      return true;
    }
    else if (property == DEBUGMESSAGES)
    {
      if (value.toString().equalsIgnoreCase("true"))
        m_debug = true;
      else
        m_debug = false;

      log("Debugging " + m_debug);
      return true;
    }
    else if (property == DEBUGMESSAGES_ALL)
    {
      if (value.toString().equalsIgnoreCase("true"))
        m_debugAll = true;
      else
        m_debugAll = false;
      log("Debugging " + m_debugAll);

      return true;
    }
    else
    {
     return super.setProperty(property, value);
    }
  }

  private void defineInfoField(Point infoLocation)
  {
    log("Searching for Info field at " + infoLocation.toString());
    Container parent = this.getParent();
    Component infoField = parent.getComponentAt(infoLocation);
    if (infoField instanceof LWCommonText)
    {
      log("Found TextField");
      m_infoField = (LWCommonText)infoField;
      m_isFound = true;
    }
    else
    {
      //Assume it's a TextArea then
      Container childContainer = (Container)infoField;
      infoField = childContainer.getComponent(0);
      if (infoField instanceof LWCommonText)
      {
        log("Found TextArea");
        m_infoField = (LWCommonText)infoField;
        m_isFound = true;
      }
      else
      {
        log("Error: Unable to find suitable Text Item to write to");
        m_isFound = false;
      }
    }

    if (m_mouseAdapter == null)
    {
      log("Creating mouse adaptor");
      m_mouseAdapter = new InfoButtonMouseAdapter();
      addMouseListener(m_mouseAdapter);
    }
  }

  /**
   * Utility function to print out a debug message to the Java Console
   * @param msg string to display, this will be prefixed with the classname of the PJC
   */
  public void log(String msg)
  {
    if(m_debug||m_debugAll)
        System.out.println(CLASSNAME + ": " + msg);
  }

  /**
   * Private class to handle user mouse actions and to switch images when the
   * user moves the mouse into and out of the button object.
   */
  class InfoButtonMouseAdapter extends MouseAdapter
  {
    /**
     * User moved the mouse over the button set the cursor and display the text
     */
    public void mouseEntered(MouseEvent me)
    {
      setCursor (new Cursor(Cursor.HAND_CURSOR));

      if (m_isFound)
      {
        m_savedText = m_infoField.getText();
        m_infoField.setText(m_text);
      }
    }

    /**
     * User moved the mouse out of the button, hide the text and restore the cursor
     */
    public void mouseExited(MouseEvent me)
    {

      if (m_isFound)
      {
        m_infoField.setText(m_savedText);
      }
    }
  }
}