package oracle.forms.demos.enhancedItems;

import oracle.forms.ui.VBean;
import oracle.forms.ui.CustomEvent;
import oracle.forms.properties.ID;
import oracle.forms.handler.IHandler;
import oracle.forms.engine.*;
import java.util.*;

/**
 * <H3><FONT COLOR="#336699">InsertReplaceIndicator JavaBeam for Oracle Forms</FONT></H3>
 * This class is a helper class for the InsertReplaceTextField and InsertReplaceTextArea classes.  When those
 * classes are put into Replace mode - say by pressing the insert key this Bean will raise a custom Item event to notify the
 * form of the change
 * The event REPLACE_TOGGLE will be raised with the Parameter REPLACE_MODE set to True or False
 * This bean could be changed to simply desplay a visual indicator byt itself and save roundtrips to
 * the form.
 * <em><b>Note:</b> This bean is not required for the operation of the InsertReplaceTextField and InsertReplaceTextArea PJCs but simply
 * provides a way of detecting toggle between modes and using that information to say update an indicator on the screen</em>
 * <br>
 * For more PJC Demos and White Papers see <a href="http://otn.oracle.com/products/forms" target="_blank">Forms on OTN</a>
 *
 * @version 9.0 06/Nov/2001<br>
 *
 * @author Duncan Mills
 */
public class InsertReplaceIndicator extends VBean implements Runnable
{
  /**
   * The property registered to send the Replace mode back to the Form.<br>
   * Sent in the :SYSTEM.CUSTOM_ITEM_EVENT_PARAMETERS
   */
  public final static ID REPLACE_MODE        = ID.registerProperty("REPLACE_MODE");

  /**
   * The property registered be sent back from the bean as a Custom Event
   */
  public final static ID REPLACE_TOGGLE      = ID.registerProperty("REPLACE_TOGGLE");

  /**
   * The classname used for debugging purposes (in the log method).
   * use getClass().getName(); if you want the
   * full package name
   */
  private final String CLASSNAME = this.getDefaultName();

  /**
   * do we want to debug for this class?  Set this to true to see debug messages.
   */
  private boolean m_debug = false;

  /**
   * Store the current value for the Replacemode - only send an event if it has changed
   */
  private boolean m_savedMode = false;


  /**
   * Hold a reference to the Handler
   */
  public static IHandler m_handler;

  /**
   * The thread used by this component
   */
  private Thread checkerThread = null;

  private boolean m_continue = false;



  /**
   * constructor
   */
  public InsertReplaceIndicator()
  {
    super();
  }

  /**
   * Method from the oracle.forms.ui.IView class and overriden from the VBean
   * base class.  This lifecycle method is called once when Forms creates an
   * instance of the JavaBean.
   *
   * @param <b>handler</b> a reference to the Handler class for this JavaBean.
   */
  public void init(IHandler handler)
  {
    m_handler = handler;
    super.init(handler);
    this.start();
  }


  /**
   * Method from the oracle.forms.ui.IView class and overriden from the VBean
   * base class.  This lifecycle method is called once when Forms destroys an
   * instance of the JavaBean.
   */
  public void destroy()
  {
    this.stop();
    super.destroy();
  }



  private void checkReplaceMode()
  {
    if (InsertReplaceHelper.getReplaceMode() != m_savedMode)
    {
      m_savedMode = InsertReplaceHelper.getReplaceMode();
      try
      {
        CustomEvent ce = new CustomEvent(m_handler,REPLACE_TOGGLE);
        m_handler.setProperty(REPLACE_MODE,"" + m_savedMode);
        super.dispatchCustomEvent(ce);
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
  }


  /**
   * Start the components actions by initialising the thread and setting it free
   */
  public void start()
  {
    log("Thread start()");
    checkerThread = new Thread(this);
    m_continue = true;
    checkerThread.start();
  }

  /**
   * Stop the component's actions by killing the thread and setting it back
   * to null for future use.
   */
  public void stop()
  {
    log("Thread stop()");
    checkerThread = null;
    m_continue = false;
  }

  /**
   * This checks the state of the replace mode every 500 ms
   */
  public void run()
  {
    try
    {
      while (m_continue)
      {
        // do the check
        checkReplaceMode();

        Thread.sleep(500);
      }
    }
    catch (InterruptedException e)
    {
      e.printStackTrace();
    }
    catch (NullPointerException npe)
    {
    }
  }


  /**
   * Utility function to print out a debug message to the Java Console.
   * @param msg string to display, this will be prefixed with the classname of the PJC
   */
  public final void log(String msg)
  {
    if(m_debug)
        System.out.println(CLASSNAME + ": " + msg);
  }
}