package oracle.forms.demos.enhancedItems;

import oracle.forms.ui.*;
import oracle.forms.properties.ID;
import oracle.forms.engine.*;
import java.awt.event.*;
import oracle.ewt.lwAWT.lwText.LWTextArea;

/**
 * <H3><FONT COLOR="#336699">InsertReplaceTextArea Pluggable Java Component for Oracle Forms</FONT></H3>
 * This class is a Pluggable Java Component for Oracle Forms, that implements
 * a method for toggling between insert and replace mode much like a text editor or a character mode application
 *
 * The class implements the following Custom properties for the programmer to
 * use:
 * <ul><li>REPLACE_MODE</li>
 *     <li>DEBUGMESSAGES</li>
 *     <li>DEBUGMESSAGES_ALL</li></ul>
 * <br>
 * For more PJC Demos and White Papers see <a href="http://otn.oracle.com/products/forms" target="_blank">Forms on OTN</a>
 *
 * @version 9.0 06/Nov/2001<br>
 *
 * @author Duncan Mills
 */
public class InsertReplaceTextArea extends VTextArea
{

  /**
   * The property registered to set the field to Replace mode although the
   * user pressing the insert key will do the same thing
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  public final static ID REPLACE_MODE        = ID.registerProperty("REPLACE_MODE");

  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on.
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  private static final ID DEBUGMESSAGES     = ID.registerProperty("DEBUGMESSAGES");
  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console for ALL INSTANCES of this PJC
   * so we can see what is going on.
   * This will generate a <u>LOT</u> of messages
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  private static final ID DEBUGMESSAGES_ALL = ID.registerProperty("DEBUGMESSAGES_ALL");

  /**
   * The classname used for debugging purposes (in the log method).
   * use getClass().getName(); if you want the
   * full package name
   */
  private final String CLASSNAME = this.getDefaultName();

  /**
   * do we want to debug for this class?  Set this to true to see debug messages.
   */
  private boolean m_debug = false;

  /**
   * do we want to debug for all instances of this class?
   */
  private static boolean m_debugAll = false;


  /**
   * A text area is actually a Text area within a scrollbox
   * so this refers to the scrollbox not the TextArea
   * The following variable will hold the actual reference
   * to the text area
   * We have to create our own subclassed version of the
   * standard text area which knows how to handle replace
   * mode
   */
  private LWTextArea m_textArea     = new InsertReplaceLWTextArea();

  /**
   * Constructor for the InsertReplaceTextArea PJC
   */

  public InsertReplaceTextArea()
  {
    super();
    // Replace the default textarea with our special one
    this.setContent(m_textArea);
  }

/**
 * Implementation of IView interface. This code in the PJC is called when
 * PL/SQL in Forms issues a GET_CUSTOM_PROPERTY call The value returned from
 * this method is returned to PL/SQL
 *
 * @param pid property ID
 * @return    value of the property
 * @see IView
 */
  public Object getProperty(ID pid)
  {
    if ( pid == REPLACE_MODE)
    {
      log("getProperty(" + InsertReplaceHelper.getPropertyName(pid) + ")");
      return "" + InsertReplaceHelper.getReplaceMode();
    }
    else
    {
      return super.getProperty(pid);
    }
  }


/**
 * Implementation of IView interface. This code in the PJC is called when
 * PL/SQL in Forms issues a SET_CUSTOM_PROPERTY call
 *
 * @param pid   property to be set
 * @param value new value of the property
 * @return      true (if the property could be set), false otherwise
 * @see IView
 */
  public boolean setProperty(ID pid, Object value)
  {
    if ( pid == REPLACE_MODE )
    {
      log("setProperty(" + InsertReplaceHelper.getPropertyName(pid) + ")");
      InsertReplaceHelper.setReplaceMode(InsertReplaceHelper.toBoolean(value,false));
      return true;
    }
    else if (pid == DEBUGMESSAGES)
    {
      m_debug = InsertReplaceHelper.toBoolean(value,false);
      log("Debugging " + m_debug);
      return true;
    }
    else if (pid == DEBUGMESSAGES_ALL)
    {
      m_debugAll = InsertReplaceHelper.toBoolean(value,false);;
      log("Debugging (All) " + m_debugAll);

      return true;
    }
    else
    {
      return super.setProperty(pid, value);
    }
  }


  /**
   * Utility function to print out a debug message to the Java Console.
   * @param msg string to display, this will be prefixed with the classname of the PJC
   */
  public final void log(String msg)
  {
    if(m_debug||m_debugAll)
        System.out.println(CLASSNAME + ": " + msg);
  }

}


