package oracle.forms.demos.enhancedItems;

import oracle.forms.ui.*;
import oracle.forms.properties.ID;
import oracle.forms.engine.*;
import java.awt.event.*;

/**
 * <H3><FONT COLOR="#336699">InsertReplaceTextField Pluggable Java Component for Oracle Forms</FONT></H3>
 * This class is a Pluggable Java Component for Oracle Forms, that implements
 * a method for toggling between insert and replace mode much like a text editor or a character mode application
 *
 * The class implements the following Custom properties for the programmer to
 * use:
 * <ul><li>REPLACE_MODE</li>
 *     <li>DEBUGMESSAGES</li>
 *     <li>DEBUGMESSAGES_ALL</li></ul>
 * <br>
 * For more PJC Demos and White Papers see <a href="http://otn.oracle.com/products/forms" target="_blank">Forms on OTN</a>
 *
 * @version 9.0 06/Nov/2001<br>
 *
 * @author Duncan Mills
 */
public class InsertReplaceTextField extends VTextField
{

  /**
   * The property registered to set the field to Replace mode although the
   * user pressing the insert key will do the same thing
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  public final static ID REPLACE_MODE        = ID.registerProperty("REPLACE_MODE");

  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on.
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  private static final ID DEBUGMESSAGES     = ID.registerProperty("DEBUGMESSAGES");
  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console for ALL INSTANCES of this PJC
   * so we can see what is going on.
   * This will generate a <u>LOT</u> of messages
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  private static final ID DEBUGMESSAGES_ALL = ID.registerProperty("DEBUGMESSAGES_ALL");

  /**
   * The classname used for debugging purposes (in the log method).
   * use getClass().getName(); if you want the
   * full package name
   */
  private final String CLASSNAME = this.getDefaultName();

  /**
   * do we want to debug for this class?  Set this to true to see debug messages.
   */
  private boolean m_debug = false;

  /**
   * do we want to debug for all instances of this class?
   */
  private static boolean m_debugAll = false;


  /**
   * Constructor for the InsertReplaceTextField PJC
   */

  public InsertReplaceTextField()
  {
    super();
  }

/**
 * Implementation of IView interface. This code in the PJC is called when
 * PL/SQL in Forms issues a GET_CUSTOM_PROPERTY call The value returned from
 * this method is returned to PL/SQL
 *
 * @param pid property ID
 * @return    value of the property
 * @see IView
 */
  public Object getProperty(ID pid)
  {
    if ( pid == REPLACE_MODE)
    {
      log("getProperty(" + InsertReplaceHelper.getPropertyName(pid) + ")");
      return "" + InsertReplaceHelper.getReplaceMode();
    }
    else
    {
      return super.getProperty(pid);
    }
  }


/**
 * Implementation of IView interface. This code in the PJC is called when
 * PL/SQL in Forms issues a SET_CUSTOM_PROPERTY call
 *
 * @param pid   property to be set
 * @param value new value of the property
 * @return      true (if the property could be set), false otherwise
 * @see IView
 */
  public boolean setProperty(ID pid, Object value)
  {

    if ( pid == REPLACE_MODE )
    {
      log("setProperty(" + InsertReplaceHelper.getPropertyName(pid) + ")");
      InsertReplaceHelper.setReplaceMode(InsertReplaceHelper.toBoolean(value,false));
      return true;
    }
    else if (pid == DEBUGMESSAGES)
    {
      m_debug = InsertReplaceHelper.toBoolean(value,false);
      log("Debugging " + m_debug);
      return true;
    }
    else if (pid == DEBUGMESSAGES_ALL)
    {
      m_debugAll = InsertReplaceHelper.toBoolean(value,false);;
      log("Debugging (All) " + m_debugAll);

      return true;
    }
    else
    {
      return super.setProperty(pid, value);
    }
  }

  /**
   * This is where the PJC does all it's work, this method overrides the
   * default keyprocessing for the underlying ewt TextComponent
   * Here, if replace mode is specified then it will replace the value with the
   * current keystroke
   * @param event the KeyEvent that is passed to us to process
   */

   protected void processKeyEvent(KeyEvent event)
   {
     if (event.getID() == KeyEvent.KEY_TYPED && InsertReplaceHelper.getReplaceMode())
     {
      log(event.paramString() + "Key Code=" + event.getKeyCode() + "KeyChar=" + new Integer(event.getKeyChar()));
      int caretPos = this.getCaretPosition();
      if (caretPos != this.getSelectionStart())
      {
        //in this case don't replace manually as the user has made a selection
        //and the default functionality will replace for us
        super.processKeyEvent(event);
      }
      else
      {
        // Don't replace if we are at the end of the string
        if (caretPos < this.getTextLength())
          //this is where the work actually happens
          //last chance saloon to exclude keysrokes
          // Backspace for instance seems to get through
          switch (event.getKeyChar())
          {
            case 8:
            case 9:
              super.processKeyEvent(event);
              break;
            default:
              this.replaceRange("" + event.getKeyChar(),caretPos,caretPos+1);
          }
        else
         super.processKeyEvent(event);
      }
     }
     else if (event.getID() == KeyEvent.KEY_PRESSED && event.getKeyCode() == KeyEvent.VK_INSERT && event.getModifiers() == 0)
     {
       log("Toggling insert/replace from keyboard");
       if (InsertReplaceHelper.getReplaceMode())
       {
         InsertReplaceHelper.setReplaceMode(false);
         log("ReplaceMode now off");
       }
       else
       {
         InsertReplaceHelper.setReplaceMode(true);
         log("ReplaceMode now on");
       }
       super.processKeyEvent(event);
     }
     else
     {
       super.processKeyEvent(event);
     }
   }


  /**
   * Utility function to print out a debug message to the Java Console.
   * @param msg string to display, this will be prefixed with the classname of the PJC
   */
  public final void log(String msg)
  {
    if(m_debug||m_debugAll)
        System.out.println(CLASSNAME + ": " + msg);
  }

}


