package oracle.forms.demos;

import oracle.forms.ui.*;
import oracle.forms.handler.IHandler;
import oracle.forms.properties.ID;
import oracle.forms.engine.*;
import java.awt.event.*;

/**
 * <H3><FONT COLOR="#336699">KeyFilter Pluggable Java Component for Oracle Forms</FONT></H3>
 * This class is a Pluggable Java Component for Oracle Forms, that implements
 * a method for only allowing defined keystrokes to be entered into a field.
 * The class also provides some additional information for the Forms programmer
 * in the form of information about what the last keypress delivered to the
 * field was and the Shift state at the time.
 *
 * The class implements the following Custom properties for the programmer to
 * use:
 * <ul><li>KEY_CODE</li>
 *     <li>KEY_CHARACTER</li>
 *     <li>KEY_MODIFIER</li>
 *     <li>FILTER_TYPE</li>
 *     <li>CUSTOM_FILTER</li>
 *     <li>DEBUGMESSAGES</li>
 *     <li>DEBUGMESSAGES_ALL</li></ul>
 * The Properties <b>KEY_CODE</b>, <b>KEY_CHARACTER</b> and <b>KEY_MODIFIER</b>
 * are read only.
 * <br>
 * For more PJC Demos and White Papers see <a href="http://otn.oracle.com/products/forms" target="_blank">Forms on OTN</a>
 *
 * @version 9.0 19/10/2001<br>
 *
 * @author Duncan Mills
 */
public class KeyFilter extends VTextField
{

  /**
   * The property registered to return the ASCII code of the last key-press.
   * Used with GET_CUSTOM_PROPERTY() from the Form
   */
  public final static ID KEYCODE        = ID.registerProperty("KEY_CODE");
  /**
   * The property registered to return the character of the last key pressed,
   * or a control key enclosed in braces. For instance F1 will set this property to {F1}
   * Used with GET_CUSTOM_PROPERTY() from the Form
   */
  public final static ID KEYCHAR        = ID.registerProperty("KEY_CHARACTER");
  /**
   * The property registered to return the modifier in force when the last key
   * was pressed. will return SHIFT, CTRL or ALT or a combination thereof
   * Used with GET_CUSTOM_PROPERTY() from the Form
   */
  public final static ID KEYMODIFIER    = ID.registerProperty("KEY_MODIFIER");
  /**
   * The property registered to return or set the current filter on the field.
   * Used with SET and GET_CUSTOM_PROPERTY() from the Form
   */
  public final static ID FILTERTYPE     = ID.registerProperty("FILTER_TYPE");
  /**
   * The property registered to return or set the custom filter on the field.
   * Used with SET and GET_CUSTOM_PROPERTY() from the Form
   */
  public final static ID CUSTOMFILTER   = ID.registerProperty("CUSTOM_FILTER");

  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on.
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  private static final ID DEBUGMESSAGES     = ID.registerProperty("DEBUGMESSAGES");
  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console for ALL INSTANCES of this PJC
   * so we can see what is going on.
   * This will generate a <u>LOT</u> of messages
   * Used with SET_CUSTOM_PROPERTY() from the Form
   */
  private static final ID DEBUGMESSAGES_ALL = ID.registerProperty("DEBUGMESSAGES_ALL");

  /**
   * The classname used for debugging purposes (in the log method).
   * use getClass().getName(); if you want the
   * full package name
   */
  private final String CLASSNAME = this.getDefaultName();


  private final static int ALPHA_FILTER        = 0;
  private final static int NUMERIC_FILTER      = 1;
  private final static int ALPHANUMERIC_FILTER = 2;
  private final static int CUSTOM_FILTER       = 3;
  private final static int NO_FILTER           = 4;


  private String    m_savedKeyChar = null;
  private char      m_filterChar = 0;
  private int       m_savedKeyCode;
  private String    m_savedModifier;
  private int       m_filterState;
  private String[]  m_filterArrays = new String[4];
  private String[]  m_filterNames = new String[5];
  private boolean   m_customFilterSet = false;

  /**
   * do we want to debug for this class?  Set this to true to see debug messages.
   */
  private boolean m_debug = false;

  /**
   * do we want to debug for all instances of this class?
   */
  private static boolean m_debugAll = false;


  /**
   * Constructor for the KeyFilter PJC
   */

  public KeyFilter() {
    super();
    try  {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    // Set up the pre-defined filters
    m_filterState = NO_FILTER;
    m_filterArrays[ALPHA_FILTER]        = new String ("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ ");
    m_filterArrays[NUMERIC_FILTER]      = new String ("1234567890.,");
    m_filterArrays[ALPHANUMERIC_FILTER] = new String ("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ 1234567890.,");
    m_filterNames[ALPHA_FILTER]         = new String("ALPHA");
    m_filterNames[NUMERIC_FILTER]       = new String("NUMERIC");
    m_filterNames[ALPHANUMERIC_FILTER]  = new String("ALPHANUMERIC");
    m_filterNames[CUSTOM_FILTER]        = new String("CUSTOM");
    m_filterNames[NO_FILTER]            = new String("NONE");

    // Hook into the keyboard
    addKeyListener(new KeyPressAdapter());
    }

/**
 * Implementation of IView interface. This code in the PJC is called when
 * PL/SQL in Forms issues a GET_CUSTOM_PROPERTY call The value returned from
 * this method is returned to PL/SQL
 *
 * @param pid property ID
 * @return    value of the property
 * @see IView
 */
  public Object getProperty(ID pid)
  {
    if ( pid == KEYCODE )
    {
      log("getProperty(KEYCODE)=" + m_savedKeyCode);
      return ""+ m_savedKeyCode;
    }
    else if ( pid == KEYCHAR )
    {
      log("getProperty(KEYCHAR)=" + m_savedKeyChar);
      return "" + m_savedKeyChar;
    }
    else if ( pid == KEYMODIFIER )
    {
      log("getProperty(KEYMODIFIER)=" + m_savedModifier);
      return m_savedModifier;
    }
    if ( pid == FILTERTYPE )
    {
      log("getProperty(FILTER_TYPE)");
      return m_filterNames[m_filterState];
    }
    else if (pid == CUSTOMFILTER)
    {
      log("getProperty(CUSTOM_FILTER)");
      if (m_customFilterSet)
      {
        log("Returning Filter");
        return m_filterArrays[CUSTOM_FILTER];
      }
      else
      {
        log("No Filter currently set returning NULL");
        return null;
      }
    }
    else
    {
      return super.getProperty(pid);
    }
  }

/**
 * Implementation of IView interface. This code in the PJC is called when
 * PL/SQL in Forms issues a SET_CUSTOM_PROPERTY call
 *
 * @param pid   property to be set
 * @param value new value of the property
 * @return      true (if the property could be set), false otherwise
 * @see IView
 */
  public boolean setProperty(ID pid, Object value)
  {
    if ( pid == FILTERTYPE )
    {
      String PropValue = value.toString();
      log("Setting FILTER_TYPE to " + PropValue);
      m_filterState = 4;
      for (int i = 0; i <= 3;i++)
      {
        if (PropValue.toUpperCase().compareTo(m_filterNames[i]) == 0)
        {
          m_filterState = i;
          break;
        }
      }
      log("Filter set to " + m_filterNames[m_filterState]);
      return true;
    }
    else if (pid == CUSTOMFILTER)
    {
      log("Setting CUSTOM_FILTER to " + value.toString());
      /*
       * Check the hashcode here as a way to check that the false
       * returned by toString is due to the value being null rather
       * than just the string "false" which could possibly be set as
       * a custom filter by the user
       */
      if ((value.toString().equals("false")) && (value.hashCode() == 1237))
      {
        log("Null custom Filter");
        m_customFilterSet = false;
      }
      else
      {
        m_filterArrays[CUSTOM_FILTER] = value.toString();
        m_customFilterSet = true;
        log("Custom Filter set");
      }
      return true;
    }
    else if (pid == DEBUGMESSAGES)
    {
      if (value.toString().equalsIgnoreCase("true"))
        m_debug = true;
      else
        m_debug = false;

      log("Debugging " + m_debug);
      return true;
    }
    else if (pid == DEBUGMESSAGES_ALL)
    {
      if (value.toString().equalsIgnoreCase("true"))
        m_debugAll = true;
      else
        m_debugAll = false;
      log("Debugging " + m_debugAll);

      return true;
    }
    else
    {
      return super.setProperty(pid, value);
    }
  }

  /**
   * Utility function to print out a debug message to the Java Console.
   * @param msg string to display, this will be prefixed with the classname of the PJC
   */
  public void log(String msg)
  {
    if(m_debug||m_debugAll)
        System.out.println(CLASSNAME + ": " + msg);
  }

  /**
    * Private class to intercept the keyboard actions and store the keystrokes
    * Away.  Also impliments the Filter functionality.
    */
  class KeyPressAdapter
  extends KeyAdapter
  {
    /**
      * Have to save away the Key in keyPresses as the key character
      * is exposed here but not in keyTyped
      */
    public void keyPressed(KeyEvent e)
    {
      int startPos = 0;
      int endPos = 0;
      String charPortion = null;

      //log("KeyPressed KeyEvent=" + e.paramString());

      //get KeyCode
      m_savedKeyCode = e.getKeyCode();

      //get Modifier if present
      startPos = e.paramString().indexOf("modifiers=");
      if (startPos > 0)
      {
        m_savedModifier = e.paramString().substring(startPos+10);
        endPos = (startPos - 1);
      }
      else
        m_savedModifier = null;

      //Now locate the KeyChar
      startPos = e.paramString().indexOf("keyCode=");
      startPos = e.paramString().indexOf(',',startPos) + 1;

      if (endPos > startPos)
      {
        charPortion = e.paramString().substring(startPos,endPos);
      }
      else
      {
        charPortion = e.paramString().substring(startPos);
      }

      if (charPortion.indexOf('=') > 0)
      {
        m_filterChar = e.getKeyChar();

        if (m_filterChar == e.CHAR_UNDEFINED||m_filterChar == e.VK_UNDEFINED) // If a modifier is pressed on it's own
          m_savedKeyChar = "{Modifier}";
        else
          m_savedKeyChar = "" + m_filterChar;
      }
      else
      {
        // save the edit name eclosed in braces
        m_filterChar = 0;
        m_savedKeyChar = "{" + charPortion + "}";
      }
    }

    /**
      * KeyTyped is where we do the filtering and replace the
      * Key Stroke if required
      */
    public void keyTyped(KeyEvent e)
    {
      if ((m_filterChar != 0) && (filterRequired(m_filterState,m_filterChar)))
      {
       e.setKeyChar(java.awt.event.KeyEvent.CHAR_UNDEFINED);
      }
    }

    /**
      * Check the Key typed against the Filter in force.  If the
      * Key is not in the string then return true so it gets
      * filtered out.
      * If no filter is in force, or the key is in the filter string
      * then return false.
      */

    private boolean filterRequired(int FilterType, int KeyChar)
    {
      if (FilterType == NO_FILTER)
      {
        return false;
      }
      else
      {
        if ((FilterType == CUSTOM_FILTER) && (!m_customFilterSet))
        {
          return false;
        }
        else if (m_filterArrays[FilterType].indexOf(KeyChar) == -1)
        {
          return true;
        }
        else
        {
          return false;
        }
      }
    }
  }
}


