package oracle.forms.jdapi.util;

import java.util.*;

import oracle.forms.jdapi.*;

/**
 * Browses the Jdapi Metadata and returns ordered lists of:<br>
 * All valid forms objects (getGenericList)<br>
 * All the possible parents for a given object (getParents)<br>
 * All the possible child objects for a given object (getChildren)<br>
 * All the properties that an object can have (getMetaProperties)<br>
 *
 * @author Mark Warner (mark.warner@oracle.com)
 * @author Dan Pusceddu (dan.pusceddu@oracle.com)
 */

public class MetaBrowser implements Comparator
{
  /**
   * Constructor - does nothing.
   */
  public MetaBrowser()
  {
  }

  /**
   * Comparator Interface to compare string lexicographically
   */
  public int compare(Object x, Object y)
  {
    return ( ((String)x).compareTo( (String)y ) );
  }
  public boolean equals(Object z)
  {
    return this.equals(z);
  }

  /**
   * Returns an enumeration of all forms object names as an alphabetically sorted
   * list of Strings.
   *
   * @return Enumeration
   */
  public Enumeration getGenericList()
  {
    Vector sortedClassNames = new Vector();

    Iterator cls = JdapiMetadata.getJdapiClasses();
    while (cls.hasNext() )
    {
      sortedClassNames.add(returnName( (Class)cls.next() ));
    }

    quickSort(sortedClassNames, this);

    return sortedClassNames.elements();
  }

  /**
   * Takes a jdapi class name and gets all of it's possible parents classes,
   * returning their types as a sorted list of Strings.
   *
   * @param String className
   * @return Enumeration
   */
  public Enumeration getParents(String className)
  {
    Class objClass = null;
    try
    {
      objClass = Class.forName("oracle.forms.jdapi." + className);
    }
    catch (ClassNotFoundException e)
    {
      return null;
    }
    Vector parentList = new Vector();

    JdapiMetaObject obj = JdapiMetadata.getJdapiMetaObject(objClass);
    Class[] parentClasses = obj.getParentClasses();
    for(int n = 0; n < parentClasses.length; n++)
    {
      parentList.add(returnName(parentClasses[n]) );
    }

    quickSort(parentList, this);

    return parentList.elements();
  }

  /**
   * Takes a jdapi class name and gets all of it's possible child classes,
   * returning their types as a sorted list of Strings.
   *
   * @param String className
   * @return Enumeration
   */
  public Enumeration getChildren(String className)
  {
    Class objClass = null;
    try
    {
      objClass = Class.forName("oracle.forms.jdapi." + className);
    }
    catch (ClassNotFoundException e)
    {
      return null;
    }

    Vector childList = new Vector();

    JdapiMetaObject obj = JdapiMetadata.getJdapiMetaObject(objClass);
    JdapiIterator iter = obj.getChildObjectMetaProperties();

    while(iter.hasNext() )
    {
      int propID = ( (JdapiMetaProperty)iter.next() ).getPropertyId();

      // hack to fix the OLT child hack!
      if (propID == JdapiTypes.JDAPI_OLT_OBJECTS_PTID) continue;
      Class propClass[] = PropertyMap.getPropertyTypes(objClass, propID);

      childList.add(returnName(propClass[0]) );
    }

    if (childList.size() > 1)
    {
      quickSort(childList, this);
    }

    return childList.elements();
  }

  /**
   * Takes a jdapi class name and gets all of it's possible metaproperties,
   * sorted by type.
   *
   * @param String className
   * @return Enumeration of PropertyContainers
   *
   * @see PropertyContainer
   */
  public Enumeration getMetaProperties(String className)
  {
System.out.println("getMetaProperties 1");
    Class objClass = null;
    try
    {
System.out.println("getMetaProperties 2");
      objClass = Class.forName("oracle.forms.jdapi." + className);
    }
    catch (ClassNotFoundException e)
    {
System.out.println("getMetaProperties 3");
      return null;
    }

System.out.println("getMetaProperties 4");
    Vector sortedMetaProperties = new Vector();

System.out.println("getMetaProperties 5");
    JdapiMetaObject obj = JdapiMetadata.getJdapiMetaObject(objClass);

System.out.println("getMetaProperties 6");
    addBooleanMetaProperties(obj, sortedMetaProperties);
System.out.println("getMetaProperties 7");
    addIntegerMetaProperties(obj, sortedMetaProperties);
System.out.println("getMetaProperties 8");
    addStringMetaProperties(obj, sortedMetaProperties);
System.out.println("getMetaProperties 9");
    addObjectMetaProperties(obj, sortedMetaProperties);
System.out.println("getMetaProperties 10");

    return sortedMetaProperties.elements();
  }



  /**
   * Quicksort implementation. Sorts Vector v using compare function defined.
   *
   * @param Vector v to sort
   * @param c Compare object used to do sorting
   */
  private void quickSort(Vector v, Comparator c)
  {
    quickSort(v, c, 0, (v.size() - 1));
  }

  /**
   * Quicksort implementation. Sorts Vector v between two indices using
   * compare function defined.
   *
   * @param Vector v to sort
   * @param c Compare object used to do sorting
   * @param start Start position in sortList to sort from
   * @param end End position in sortList to sort to
   */
  private void quickSort(Vector v, Comparator c, int start, int end)
  {
    if (v.size() > 1)
    {
      if(start < 0 || start >= v.size())
      {
        throw new IndexOutOfBoundsException("start");
      }
      if(end < 0 || end >= v.size())
      {
        throw new IndexOutOfBoundsException("end");
      }
      if(start > end)
      {
        throw new IllegalArgumentException("start must be <= end");
      }

      int i = start;
      int j = end;
      Object x = v.elementAt((start + end) / 2);

      do
      {
        while((c.compare((String)v.elementAt(i), (String)x) < 0) && (i < end))
        {
          ++i;
        }
        while((c.compare((String)x, (String)v.elementAt(j)) < 0) && (j > start))
        {
          --j;
        }

        if(i <= j)
        {
          Object y = v.elementAt(i);
          v.setElementAt(v.elementAt(j), i);
          v.setElementAt(y, j);
          ++i;
          --j;
        }
      } while(i <= j);

      if(start < j)
      {
        quickSort(v, c, start, j);
      }
      if(i < end)
      {
        quickSort(v, c, i, end);
      }
    }
  }

  /**
   * Returns the class name without full qualification as a String.
   *
   * @param Class x
   * @return String
   */
  private String returnName(Class x)
  {
    String className = x.getName();
    return className.substring(className.lastIndexOf(".") + 1);
  }

  /**
   * Takes a JdapiMetaObject and adds all of its boolean meta properties to a
   * given vector in PropertyContainers.
   *
   * @param JdapiMetaObject obj
   * @param Vector sortedMetaProperties - vector of PropertyContainers.
   */
  private void addBooleanMetaProperties(JdapiMetaObject obj, Vector sortedMetaProperties)
  {
    // iterate through the boolean meta properties.
    JdapiIterator iter = obj.getBooleanMetaProperties();

    while(iter.hasNext() )
    {
      addMetaProperty((JdapiMetaProperty)iter.next(), sortedMetaProperties);
    }
  }

  /**
   * Takes a JdapiMetaObject and adds all of its integer meta properties to a
   * given vector in PropertyContainers.
   *
   * @param JdapiMetaObject obj
   * @param Vector sortedMetaProperties - vector of PropertyContainers.
   */
  private void addIntegerMetaProperties(JdapiMetaObject obj, Vector sortedMetaProperties)
  {
    // iterate through the integer meta properties.
    JdapiIterator iter = obj.getIntegerMetaProperties();

    while(iter.hasNext() )
    {
      addMetaProperty((JdapiMetaProperty)iter.next(), sortedMetaProperties);
    }
  }

  /**
   * Takes a JdapiMetaObject and adds all of its string meta properties to a
   * given vector in PropertyContainers.
   *
   * @param JdapiMetaObject obj
   * @param Vector sortedMetaProperties - vector of PropertyContainers.
   */
  private void addStringMetaProperties(JdapiMetaObject obj, Vector sortedMetaProperties)
  {
    // iterate through the string meta properties.
    JdapiIterator iter = obj.getStringMetaProperties();

    while(iter.hasNext() )
    {
      addMetaProperty((JdapiMetaProperty)iter.next(), sortedMetaProperties);
    }
  }

  /**
   * Takes a JdapiMetaObject and adds all of its object meta properties to a
   * given vector in PropertyContainers.
   *
   * @param JdapiMetaObject obj
   * @param Vector sortedMetaProperties - vector of PropertyContainers.
   */
  private void addObjectMetaProperties(JdapiMetaObject obj, Vector sortedMetaProperties)
  {
    // iterate through the object meta properties.
    JdapiIterator iter = obj.getObjectMetaProperties();

    while(iter.hasNext() )
    {
      addMetaProperty((JdapiMetaProperty)iter.next(), sortedMetaProperties);
    }
  }

  /**
   * Takes a JdapiMetaProperty creates a PropertyContainer for it and adds it to
   * a given vector.
   *
   * @param JdapiMetaProperty prop
   * @param Vector sortedMetaProperties - vector of PropertyContainers.
   */
  private void addMetaProperty(JdapiMetaProperty prop, Vector sortedMetaProperties)
  {
    PropertyContainer container = new PropertyContainer();
    int pnum = prop.getPropertyId();

    container.setPropertyNum(pnum);
    container.setPropertyName(JdapiMetaProperty.getPropertyName(pnum) );
    container.setPropertyType(returnName(JdapiMetaProperty.getPropertyType(pnum)) );
    container.setPropertyDesc(JdapiMetaProperty.getPropertyDescription(pnum) );

    int[] intVals = prop.getAllowedValues();
    if(null != intVals)
    {
      HashMap vals = new HashMap();
      for(int n = 0; n < intVals.length; n++)
      {
        try
        {
          vals.put(new Integer(intVals[n]), JdapiMetaProperty.getPropertyValueName(pnum,intVals[n]));
        }
        catch(JdapiException e)
        {
          vals.put(new Integer(intVals[n]), e.getMessage());
        }
      }
      container.setAllowedValues(vals);
    }
    container.allowGet(prop.allowGet() );
    container.allowSet(prop.allowSet() );

    sortedMetaProperties.add(container);
  }

}

