/* Copyright (c) Oracle Corporation 2000.  All Rights Reserved. */

package oracle.forms.demos;

import oracle.forms.ui.*;
import oracle.forms.properties.*;

import java.awt.event.*;
import java.awt.*;

/**
 *
 * <H3><FONT COLOR="003399">Oracle Forms Server PJC Example</FONT></H3>
 *
 * This is a Forms PJC which subclasses off of the standard Forms Textfield.  What
 * this means is that it is possible to modify some of the behaviour (or add it!) of
 * the standard Forms textfield component but DO NOT have to worry about all the
 * other functionality - it is obtained for free so to speak ...  If this interest
 * you, check out the Using Java in Oracle Forms Applications published on the
 * OTN website for more details on this topic.
 *
 * @author sbutton 03/16/2000
 * @version 2.0
 */
public class ModCursor
       extends VTextField
{

  /**
   * Register the property used from PL/SQL to pass in the name of the
   * cursor to use when the mouse moves over the component.
   */
  public static final ID DOCURSOR = ID.registerProperty("DOCURSOR");
  /**
   * Store the standard cursor when we need to switch back when the mouse skadoodles
   */
  private Cursor std;
  /**
   * Store the modified cursor to use when the mouse moves over the component
   */
  private Cursor mod;
  /**
   * Store the modified cursor name as a string
   */
  private String cursorType;

  /**
   * Construct the ModCursor class, based on the <code>oracle.forms.ui.VTextField</code>
   * base class.  Call the jbInit method to perform most of the (small) amount of
   * initialisation code.
   */
  public ModCursor()
  {
    super();
    try
    {
      jbInit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * The method from the oracle.forms.ui.IView interface which allows the
   * PJC to receive property settings from a running Form.  In this case,
   * all I care about is the DOCURSOR property since this indicates the
   * style of cursor I should use.  All other properties are handled by the
   * superclass <code>oracle.forms.ui.VTextField</code>.
   *
   * @param pid the property to be set
   * @param value the value to set the specified property to
   * @return the result of the setting operaration, true for success, false for fail
   */
  public boolean setProperty(ID pid, Object value)
  {
    String val;
    // is the property the one I want - DOCURSOR?
    if(pid == DOCURSOR)
    {
      val = (String)value;
      cursorType = val;

      // check the value to see if it's something I know about and then
      // set the cursor to that value if it is!
      if(val.equalsIgnoreCase("HAND"))
        mod = new Cursor(Cursor.HAND_CURSOR);
      else if(val.equalsIgnoreCase("CROSSHAIR"))
        mod = new Cursor(Cursor.CROSSHAIR_CURSOR);
      else if(val.equalsIgnoreCase("WAIT"))
        mod = new Cursor(Cursor.WAIT_CURSOR);
      else if(val.equalsIgnoreCase("MOVE"))
        mod = new Cursor(Cursor.MOVE_CURSOR);
      else if(val.equalsIgnoreCase("DEFAULT"))
        mod = std;
      return true;
    }
    else
    {
      // OK, it's nothing I care about, let the superclass take care
      // of everything else and I'll kick back, relax and have a beer!
      return super.setProperty(pid, value);
    }
  };

  /**
   * The method from the <code>oracle.forms.ui.IView</code> interface which allows the
   * PJC to return the current settings of a property to a running Form.  In this case,
   * all I care about is the DOCURSOR property since this indicates the
   * style of cursor I am using use.  All other properties are handled by the
   * superclass <code>oracle.forms.ui.VTextField</code>.
   *
   * @param pid the property identitier which indicates the property to be accessed
   * @return the value of the specific property
   */
  public Object getProperty(ID pid)
  {
    // do I care about it?
    if(pid == DOCURSOR)
    {
      // yep, return the cursortype string to the form
      return cursorType;
    }
    else
    {
      // nope, let the superclass take care of it for me!
      return super.getProperty(pid);
    }
  }

  /**
   * The JDeveloper required initialisation method.  This is called once at
   * during the class instantiation phase.
   */
  private void jbInit() throws Exception
  {
    // Obtain the default cursor value used by the superclass and store this
    // so we can reset it when the mouse leaves.
    std = this.getCursor();
    mod = std;

    // Here we're adding a mouse listener - this piece of code will get called whenever
    // the Java component notices that the mouse has moved into it or when the mouse
    // has moved out of it.  We're using an anonymous class here which makes it a little
    // cleaner with less classes to have to worry about.  We subclassed from the
    // standard AWT MouseAdapter so only need to provide those methods we care about
    // rather than the entire interface .... much easier.
    this.addMouseListener(new MouseAdapter()
    {
      /**
       * Handle the mouse entering the component.  This gets called WHENEVER the
       * mouse enters the component.  The cursor is set to the value specified
       * by the user.
       *
       * @param me the mouse event details
       */
      public void mouseEntered(MouseEvent me)
      {
        // set the cursor to user specified one
        setCursor(mod);
      }

      /**
       * Handle the mouse exiting the component.  This gets called WHENEVER the
       * mouse exits the component.  The cursor is set back to the original cursor
       * used by the superclass.
       *
       * @param me the mouse event details
       */
      public void mouseExited(MouseEvent me)
      {
        // set the cursor to it's original value
        setCursor(std);
      }
    });
  }

  //
  // that's all folks ......
  //
}

