// Copyright (c) 2001 Oracle Corp
package oracle.forms.demos.enhancedItems;

import oracle.forms.ui.*;
import oracle.forms.properties.ID;
import java.util.StringTokenizer;

/**
 * <H3><FONT COLOR="#002299">Pluggable Java Components for Oracle Forms</FONT></H3>
 * This class is a Pluggable Java Component for Oracle Forms, that implements
 * a method for allowing multi-select on a TList for use in Picker controls etc.
 *
 * The class implements the following Custom properties for the programmer to
 * use on a Tlist:
 * <ul><li>ENABLE_LIST_MULTISELECT</li>
 *     <li>LIST_SELECTION</li>
 *     <li>CLEAR_LIST_SELECTION</li></ul>
 *
 * @version 1.0 24/June/2001 created<br>
 *
 * @author Duncan Mills
 */
public class MultiSelectTList extends VTList
{

  /**
   * The property registered to enable Multi Select capability in a TList
   * TLIST
   */
  public final static ID ENABLE_LIST_MULTISELECT = ID.registerProperty("ENABLE_LIST_MULTISELECT");
  /**
   * The property registered to Get/Set the selected values in a multiselect
   * TLIST
   */
  public final static ID LIST_SELECTION = ID.registerProperty("LIST_SELECTION");

  /**
   * The property registered to clear the current selection in a multiselect
   * TLIST
   */
  public final static ID CLEAR_LIST_SELECTION = ID.registerProperty("CLEAR_LIST_SELECTION");

  /**
   * The property registered to Enable or Disable debugging on the PJC
   */
  public final static ID DEBUGMESSAGES = ID.registerProperty("DEBUGMESSAGES");

  /**
   * the classname used for debugging purposes
   * use getClass().getName(); if you want the
   * full package name
   */
  private final String CLASSNAME = this.getDefaultName();

  /**
   * do we want to debug for this class?  Set this to true to see debug messages.
   */
  private boolean m_debug = false;


  /**
   * Constructs a new instance.
   */
  public MultiSelectTList()
  {
    super();
  }

 /**
 * Implimentation of setProperty in the IView interface
 *
 * @param pid   property to be set
 * @param value new value of the property
 * @return      true (if the property could be set), false otherwise
 * @see IView
 */
   public boolean setProperty(ID pid, Object value)
  {
    if (pid == ENABLE_LIST_MULTISELECT)
    {
      log("Setting ENABLE_LIST_MULTISELECT to " + value.toString());
      this.setMultipleMode(toBoolean(value,false));
    }
    else if (pid == LIST_SELECTION)
    {
      log("Setting list selection to " + value.toString());
      this.deselectAll();
      StringTokenizer tok = new StringTokenizer(value.toString(),",");
      while (tok.hasMoreElements())
      {
        try
        {
        this.select(Integer.parseInt(tok.nextToken())-1);
        } catch (NumberFormatException e)
          {
            return false;
          }
      }
    }
    else if (pid == CLEAR_LIST_SELECTION)
    {
      log("Clearing list selection");
      this.deselectAll();
    }
    else if (pid == DEBUGMESSAGES)
    {
      m_debug = toBoolean(value,false);
    }
    else
    {
      return super.setProperty(pid, value);
    }
    return true;
  }

 /**
 * Implimentation of getProperty in the IView interface
 *
 * @param pid   property to be set
 * @return      the value of the property
 * @see IView
 */
  public Object getProperty(ID pid)
  {
    if (pid == LIST_SELECTION)
    {
      if (this.getSelectedIndexes().length > 0)
      {
        StringBuffer returnList = new StringBuffer();
        for (int i = 0; i < this.getSelectedIndexes().length; i++)
        {
          returnList.append((this.getSelectedIndexes()[i] + 1) + ",");
        }
        return returnList.toString().substring(0,(returnList.length()-1));
      }
      else
      {
        return new String("");
      }
    }
    else
    {
      return super.getProperty(pid);
    }
  }

  /**
   * Converts a string value containing TRUE or FALSE
   * as a string to the equivilent boolean
   *
   * @param value String containing the words True or false in any case or the Forms Constants PROPERTY_TRUE/PROPERTY_FALSE
   * @return boolean value of the value
   */

  protected final static boolean toBoolean(Object value, boolean defaultValue)
  {
     boolean RC = defaultValue;
     if (value.toString().equalsIgnoreCase("false")||value.toString().equals("5"))
        RC = false;
     else if (value.toString().equalsIgnoreCase("true")||value.toString().equals("4"))
        RC = true;
     return RC;
  }


  /**
   * Utility function to print out a debug message to the Java Console
   * @param msg string to display, this will be prefixed with the classname of the PJC
   */
  public void log(String msg)
  {
    if(m_debug)
        System.out.println(CLASSNAME + ": " + msg);
  }
}
